      !
C     ! Program for the CP exercise 3.1
C     ! Interpolation of the Bessel function J_0(z)
C     ! Employs Neville's algorithm for polynomial interpolation
C     ! 
C     ! Last change: 28.05.2003
C     ! Matthias Jamin
      !
      Program Bessel_Interpolate
      Implicit Double Precision (A-z)
      Integer i
      Dimension J0(10), xint(6), yint(6)
      External funcd
      !
      Data (J0(i),i=3,8)
     . / -0.2600519549019334376242D0,
     .   -0.3971498098638473722866D0,
     .   -0.1775967713143383043474D0,
     .    0.1506452572509969316623D0,
     .    0.3000792705195555966503D0,
     .    0.1716508071375539060909D0 / 
      !
C     ! Check accuracy of the different numbers
C     ! "besj0" - internal implementation of J_0(z)
      !
C     Do 10 i = 1,10
C  10 Write (*,*) i, J0(i), besj0(dble(i))-J0(i),
C    .               besselJ0(dble(i))-J0(i)
      !
      !
C     ! Initialise x and y values for the cubic interpolation
      !
      Do 20 i = 1,4
      xint(i) = 3.D0+dble(i)
   20 yint(i) = J0(i+3)
      !
C     ! Perform the cubic interpolation
      !
      Call polint(xint,yint,5.5D0,J05p5,dy,4)
      !
C     ! Compare the interpolation for J_0(5.5) and the exact result
      !
      Write (*,*) ""
      Write (*,*) J05p5, besselJ0(5.5D0), besselJ0(5.5D0)-J05p5
      !
      !
C     ! Calculate root close to z = 5.5
      !
      root = rtnewt(funcd,5.5D0,6.D0,1.D-14)
      Call polint(xint,yint,root,J0root,dy,4)
      !
      Write (*,*) ""
      Write (*,*) root, J0root
      !
      !
C     ! Initialise x and y values for the 5th order interpolation
      !
      Do 30 i = 1,6
      xint(i) = 2.D0+dble(i)
   30 yint(i) = J0(i+2)
      !
C     ! Perform the 5th order interpolation
      !
      Call polint(xint,yint,5.5D0,J05p5,dy,6)
      !
C     ! Compare the interpolation for J_0(5.5) and the exact result
      !
      Write (*,*) ""
      Write (*,*) J05p5, besselJ0(5.5D0), besselJ0(5.5D0)-J05p5
      !
      !
C     ! Perform the linear interpolation
      !
      J05p5 = (J0(6)-J0(5))*5.5D0 + (6.D0*J0(5)-5.D0*J0(6))
      !
      Write (*,*) ""
      Write (*,*) J05p5, besselJ0(5.5D0), besselJ0(5.5D0)-J05p5
      !
      !
      Stop
      End
C----------------------------------------------------------------------
      !
C     ! Series expansion for the Bessel function J_0(z)
      !
      Double Precision Function besselJ0(z)
      Implicit Double Precision (A-z)
      Integer k
      !
      fac = 1.0D0
      old = 1.0D0
      !
      Do 10 k=1,40
      fac = k*fac
      besselJ0 = old + (-1)**k/(fac)**2*(z/2.D0)**(2*k)
      If ((besselJ0-old).eq.0.D0) Return
   10 old = besselJ0
      !
      Pause 'Series has not yet converged'
      !
      End
C----------------------------------------------------------------------
      !
C     ! Series expansion for the derivative of the Bessel function J_0'(z)
      !
      Double Precision Function besselJ0p(z)
      Implicit Double Precision (A-z)
      Integer k
      !
      fac = 1.0D0
      old = 0.0D0
      !
      Do 10 k=1,40
      fac = k*fac
      besselJ0p = old + (-1)**k*k/(fac)**2*(z/2.D0)**(2*k-1)
      If ((besselJ0p-old).eq.0.D0) Return
   10 old = besselJ0p
      !
      Pause 'Series has not yet converged'
      !
      End
C----------------------------------------------------------------------
      !
C     ! Implementation of the Neville algorithm for polynomial interpolation
      !
      Subroutine polint(xa,ya,x,y,dy,n)
      Implicit Double Precision (A-z)
      Integer i, n, ns, nmax
      Parameter (nmax=10)
      Dimension xa(n), ya(n), c(nmax), d(nmax)
      !
      ns=1
      dif = dabs(x-xa(1))
      Do 11 i=1,n
         dift = dabs(x-xa(i))
         If (dift.lt.dif) Then
            ns = i
            dif = dift
         End If
         c(i) = ya(i)
         d(i) = ya(i)
   11 Continue
      !
      y = ya(ns)
      ns = ns-1
      Do 13 m=1,n-1
         Do 12 i=1,n-m
            ho = xa(i)-x
            hp = xa(i+m)-x
            w = c(i+1)-d(i)
            den = ho-hp
            If (den.eq.0.D0) Pause 'Failure in polint.'
            den = w/den
            d(i) = hp*den
            c(i) = ho*den
   12    Continue
         If (2*ns.lt.n-m) Then
            dy = c(ns+1)
         Else
            dy = d(ns)
            ns = ns-1
         End If
         y = y+dy
   13 Continue
      !
      Return
      End
C-----------------------------------------------------------------------
      !
C     ! Implementation of the Newton-Raphson algorithm
      !
      Double Precision Function rtnewt(funcd,x1,x2,xacc)
      Implicit Double Precision (A-z)
      Integer j, JMAX
      External funcd
      Parameter (JMAX=20)
      !
C     ! Initial guess for the root
      !
      rtnewt = 0.5D0*(x1+x2)
      !
C     ! Iteration loop up to JMAX iterations
      !
      Do 10 j=1,JMAX
         Call funcd(rtnewt,f,df)
         dx=f/df
         rtnewt = rtnewt-dx
         If ((x1-rtnewt)*(rtnewt-x2).lt.0.) Pause
     .'rtnewt jumped out of brackets'
         If (dabs(dx).lt.xacc) Return
   10 Continue
      !
      Pause 'rtnewt exceeded maximum iterations'
      !
      End
C----------------------------------------------------------------------
      !
C     ! Subroutine which provides the function to
C     ! be solved as well as its first derivative
      !
      Subroutine funcd(z,f,df)
      Implicit Double Precision (A-z)
      !
C     ! Function for which the root shall be calculated
      !
      f = besselJ0(z)
      !
C     ! First derivative of the function
      !
      df = besselJ0p(z)
      !
      Return
      End
C----------------------------------------------------------------------

