      !
C     ! Exercise 10.1 of my CP lecture SS 2003
C     ! Solution of the quantum mechanical anharmonic
C     ! oscillator with the Cash-Karp Runge-Kutta method
C     ! Last change: 17.7.2003
C     ! Matthias Jamin
      !
      Program QMAnharmonicPotential
      Implicit Double Precision (A-z)
      Integer i, n, ngau, nvar, nok, nbad, parity
      !
      Parameter (ngau = 284, nvar=2)
      Dimension psi(ngau), xx(ngau), wx(ngau)
      Dimension ystart(nvar)
      !
      Common /Para/ hq2, al, be, EE, parity, x0, psix0, psipx0
      Data hq2, al, be /7.6199634D0, 0.5D0, 0.25D0/
      !
C     ! Set principal quantum number n = 0, 1, 2
      !
      n = 0
      !
C     ! Starting x value where we assume psi(x_0) = 0
      !
      x0 = -6.D0
      psix0  = 0.D0
      !
C     ! To get the first rough idea where the eigenstates are
C     ! This yields e_min and e_max to be used in the call of rtsafe
      !
C     parity =  1
C     Do 10 e = 1.0D0,20.0D0,0.1D0
C     Call funcd(e,f,df)
C  10 Write (*,*) e, f
C     Stop
      !
      !
C     ! Distinguish the bound states for n = 0, 1, 2
C     ! Calculate the bound state energies E_n
C     ! Also set the derivative psi'(x_0) such that psi(x)
C     ! is normalised to 1. The values were calculated below
      !
      If (n.eq.0) Then
         parity =  1
         psipx0 = 0.129404852571D-07
         En = rtsafe(2.0D0,2.1D0,1.D-9)
      Else If (n.eq.1) Then
         parity = -1
         psipx0 = 0.534527565295D-07
         En = rtsafe(6.9D0,7.0D0,1.D-9)
      Else If (n.eq.2) Then
         parity =  1
         psipx0 = 0.182495409507D-06
         En = rtsafe(12.9D0,13.0D0,1.D-9)
      End If
      !
C     ! These are the resulting bound state energies
      Write (*,FMT='(I5, F16.9)') n, En
      !
C     E0 =  2.047462163D0
C     E1 =  6.903346076D0
C     E2 = 12.954702159D0
      !
      !
C     ! Calculate the integral from x_0 to 0
C     ! in order to normalise the wave function
      !
      SUM = 0.D0
      Call gauleg(x0,0.D0,xx,wx,ngau)
      !
      Do 20 i = 1,ngau
      ystart(1) = psix0
      ystart(2) = psipx0
      Call odeint(ystart,nvar,x0,xx(i),1.D-9,1.D-3,0.D0,nok,nbad)
      psi(i) = ystart(1)
   20 SUM = SUM + 2.D0*psi(i)**2*wx(i)
      !
      Write (6,FMT='(2E20.12)') SUM, psipx0/dsqrt(SUM)
      !
      !
C     ! Write out resulting wave function
      !
      Open (Unit=8, File='ex10.1.out')
      Write (8,FMT='(2F16.9)') x0, 0.D0
      !
      Do 30 i = 1,ngau
   30 Write (8,FMT='(2F16.9)') xx(i), psi(i)
      !
      Close (8)
      !
      Stop
      End
C----------------------------------------------------------------------
      ! 
C     ! For a given energy e, calculate the logarithmic derivative
C     ! of the wave function psi(x) (even parity) or its inverse
C     ! (odd parity). The roots of these equations determine the
C     ! bound state energies E_n.
      !
      Subroutine funcd(e,f,df)
      Implicit Double Precision (A-z)
      Integer nok, nbad, nvar, parity
      !
      Parameter (nvar=2)
      Dimension ystart(nvar)
      Common /Para/ hq2, al, be, EE, parity, x0, psix0, psipx0
      !
C     ! Trial bound state energy
      !
      EE = e
      !
C     ! Initial conditions for the DEQ
      !
      ystart(1) = psix0
      ystart(2) = psipx0
      !
      Call odeint(ystart,nvar,x0,0.D0,1.D-9,1.D-2,0.D0,nok,nbad)
      !
C     ! Distinguish between even and odd parity states
      !
      If (parity.eq.1) Then
         f = ystart(2)/ystart(1)
         df = -2.D0/hq2*EE-f*f
      Else
         f = ystart(1)/ystart(2)
         df = 1.D0+2.D0/hq2*EE*f*f
      End If
      !
C     Write (*,*) EE, f, df, nok, nbad
      !
      Return
      End
C----------------------------------------------------------------------
      !
C     ! Calculates the right-hand side derivatives
C     ! for the Schroedinger equation with anharmonic potential
      !
      Subroutine derivs(x,y,dxdy)
      Implicit Double Precision (A-z)
      Integer parity
      !
      Parameter (nvar=2)
      Dimension y(nvar), dxdy(nvar)
      Common /Para/ hq2, al, be, EE, parity, x0, psix0, psipx0
      !
      dxdy(1) = y(2)
      dxdy(2) = 2.D0/hq2*(al*x**2+be*x**4-EE)*y(1)
      !
      Return
      End
C----------------------------------------------------------------------
      !
C     ! Runge-Kutta driver with adaptive stepsize control. Integrate the
C     ! starting values ystart(1:nvar) from x1 to x2 with accuracy eps,
C     ! storing intermediate results in the common block /path/. h1 should
C     ! be set as a guessed first stepsize, hmin as the minimum allowed
C     ! stepsize (can be zero). On output, nok and nbad are the number of
C     ! good and bad (but retried and fixed) steps taken, and ystart is
C     ! replaced by values at the end of the integration interval. derivs
C     ! is the user-supplied subroutine for calculating the right-hand
C     ! side derivative, while rkqs is the name of the stepper routine to
C     ! be used. /path/ contains its own information about how often an
C     ! intermediate value is to be stored.
      !
      Subroutine odeint(ystart,nvar,x1,x2,eps,h1,hmin,nok,nbad)
      Implicit Double Precision (A-z)
      Integer i, kmax, kount, nbad, nok, nvar, nstp, KMAXX, MAXSTP, NMAX
      Dimension ystart(nvar)
C     External derivs, rkqs
      Parameter (MAXSTP=10000, NMAX=50, KMAXX=2000, TINY=1.D-30)
      Dimension dydx(NMAX),xp(KMAXX),y(NMAX),yp(NMAX,KMAXX),yscal(NMAX)
      Common /path/ kmax, kount, dxsav, xp, yp
      !
      x = x1
      h = dsign(h1,x2-x1)
      nok = 0
      nbad = 0
      kount = 0
      !
      Do 11 i = 1,nvar
        y(i) = ystart(i)
   11 Continue
      !
C     ! Assures storage of first step. 
      !
      If (kmax.gt.0) xsav = x-2.D0*dxsav
      !
C     ! Take at most MAXSTP steps.
      !
      Do 16 nstp = 1,MAXSTP
         Call derivs(x,y,dydx)
      !
C     ! Scaling used to monitor accuracy.
C     ! This general-purpose choice can be modified if need be.
      !
         Do 12 i = 1,nvar
            yscal(i) = dabs(y(i))+dabs(h*dydx(i))+TINY
   12    Continue
         If (kmax.gt.0) Then
            If (dabs(x-xsav).gt.dabs(dxsav)) Then
               If (kount.lt.kmax-1) Then
                  kount = kount+1
                  xp(kount) = x
                  Do 13 i = 1,nvar
                     yp(i,kount) = y(i)
   13             Continue
                  xsav = x
               End If
            End If
         End If
      !
C     ! If stepsize can overshoot, decrease.
      !
         If ((x+h-x2)*(x+h-x1).gt.0.D0) h = x2-x
         Call rkqs(y,dydx,nvar,x,h,eps,yscal,hdid,hnext)
         If (hdid.eq.h) Then
            nok = nok+1
         Else
            nbad = nbad+1
         End If
      !
C     ! Are we done?
      !
         If ((x-x2)*(x2-x1).ge.0.D0) Then
            Do 14 i = 1,nvar
               ystart(i) = y(i)
   14       Continue
      !
C     ! Save final step
      !
            If (kmax.ne.0) Then
               kount = kount+1
               xp(kount) = x
               Do 15 i = 1,nvar
                  yp(i,kount) = y(i)
   15          Continue
            End If
      !
C     ! Normal exit
      !
            Return
         End If
         If (dabs(hnext).lt.hmin) Pause
     . 'stepsize smaller than minimum in odeint'
         h = hnext
   16 Continue
      Pause 'too many steps in odeint'
      Return
      End
C----------------------------------------------------------------------
      !
C     ! Fifth order Runge-Kutta step with monitoring of local truncation
C     ! error to ensure accuracy and adjust stepsize. Input are the
C     ! dependent variable vector y(1:n) and its derivative dydx(1:n) at
C     ! the starting values of the independent variable x. Also input are
C     ! the stepsize to be attempted htry, the required accuracy eps, and
C     ! the vector yscal(1:n) against which the error is scaled. On output,
C     ! y and x are replaced by their new values, hdid is the stepsize
C     ! that was actually accomplished, and hnext is the estimated next
C     ! stepsize. derivs is the user-supplied subroutine that computes
C     ! the right-hand side derivatives.
      !
      Subroutine rkqs(y,dydx,n,x,htry,eps,yscal,hdid,hnext)
      Implicit Double Precision (A-z)
      INTEGER i, n, NMAX
      Dimension dydx(n), y(n), yscal(n)
C     External derivs
      !
C     ! Maximum number of equations
      !
      Parameter (NMAX=50)
      Dimension yerr(NMAX), ytemp(NMAX)
      Parameter (SAFETY=.9D0,PGROW=-.2D0,PSHRNK=-.25D0,ERRCON=1.89D-4)
      !
C     ! Set stepsize to initial trial value and take the step
      !
      h = htry
    1 Call rkck(y,dydx,n,x,h,ytemp,yerr)
      !
C     ! Evaluate accuracy
      !
      errmax = 0.D0
      Do 11 i = 1,n
         errmax = max(errmax,dabs(yerr(i)/yscal(i)))
   11 Continue
      !
C     ! Scale relative to required tolerance
      !
      errmax = errmax/eps
      If (errmax.gt.1.D0) Then
      !
C     ! Truncation error to large, reduce stepsize.
      !
         htemp = SAFETY*h*(errmax**PSHRNK)
         h = dsign(max(dabs(htemp),0.1D0*dabs(h)),h)
         xnew = x+h
         If (xnew.eq.x) pause 'stepsize underflow in rkqs'
      !
C     ! For another try
      !
         Goto 1
      !
C     ! Step succeeded. Compute size of next step
      !
      Else
         If (errmax.gt.ERRCON) Then
            hnext = SAFETY*h*(errmax**PGROW)
         Else
C     !     No more than a factor of 5 increase.
            hnext = 5.D0*h
         End If
         hdid = h
         x = x+h
         Do 12 i = 1,n
            y(i) = ytemp(i)
   12    Continue
         Return
      !
      End If
      End
C----------------------------------------------------------------------
C     ! Given values for n variables y and their derivatives dydx known
C     ! at x, use the fifth order Cash-Karp Runge-Kutta method to advance
C     ! the solution over an interval h and return the incremental
C     ! variables as yout. Also return an estimate of the local truncation
C     ! error in yout using the embedded fourth order method. The user
C     ! supplies the subroutine derivs(x,y,dydx), which returns the
C     ! derivatives dydx at x.
      !
      Subroutine rkck(y,dydx,n,x,h,yout,yerr)
      Implicit Double Precision (A-z)
      Integer i, n, NMAX
      Dimension dydx(n), y(n), yerr(n), yout(n)
C     External derivs
      !
C     ! Maximum number of equations
      !
      Parameter (NMAX=50)
      Dimension ak2(NMAX), ak3(NMAX), ak4(NMAX), ak5(NMAX), ak6(NMAX),
     .          ytemp(NMAX)
      Parameter (A2=0.2D0, A3=0.3D0, A4=0.6D0, A5=1.0D0, A6=0.875D0,
     . B21=0.2D0, B31=3.D0/40.D0, B32=9.D0/40.D0, B41=0.3D0,
     . B42=-0.9D0, B43=1.2D0, B51=-11.D0/54.D0, B52=2.5D0,
     . B53=-70.D0/27.D0, B54=35.D0/27.D0, B61=1631.D0/55296.D0,
     . B62=175.D0/512.D0, B63=575.D0/13824.D0, B64=44275.D0/110592.D0,
     . B65=253.D0/4096.D0, C1=37.D0/378.D0, C3=250.D0/621.D0,
     . C4=125.D0/594.D0, C6=512.D0/1771.D0, DC1=C1-2825.D0/27648.D0,
     . DC3=C3-18575.D0/48384.D0, DC4=C4-13525.D0/55296.D0,
     . DC5=-277.D0/14336.D0, DC6=C6-0.25D0)
      !
      Do 11 i = 1,n
         ytemp(i) = y(i)+B21*h*dydx(i)
   11 Continue
      Call derivs(x+A2*h,ytemp,ak2)
      !
      Do 12 i = 1,n
         ytemp(i) = y(i)+h*(B31*dydx(i)+B32*ak2(i))
   12 Continue
      Call derivs(x+A3*h,ytemp,ak3)
      !
      Do 13 i = 1,n
         ytemp(i) = y(i)+h*(B41*dydx(i)+B42*ak2(i)+B43*ak3(i))
   13 Continue
      Call derivs(x+A4*h,ytemp,ak4)
      !
      Do 14 i = 1,n
         ytemp(i)=y(i)+h*(B51*dydx(i)+B52*ak2(i)+B53*ak3(i)+B54*ak4(i))
   14 Continue
      Call derivs(x+A5*h,ytemp,ak5)
      !
      Do 15 i = 1,n
         ytemp(i) = y(i)+h*(B61*dydx(i)+B62*ak2(i)+B63*ak3(i)+
     .              B64*ak4(i)+B65*ak5(i))
   15 Continue
      !
      Call derivs(x+A6*h,ytemp,ak6)
      Do 16 i = 1,n
         yout(i) = y(i)+h*(C1*dydx(i)+C3*ak3(i)+C4*ak4(i)+C6*ak6(i))
   16 Continue
      !
      Do 17 i = 1,n
         yerr(i) = h*(DC1*dydx(i)+DC3*ak3(i)+DC4*ak4(i)+DC5*ak5(i)+
     .             DC6*ak6(i))
   17 Continue
      !
      Return
      End
C----------------------------------------------------------------------
      Double Precision Function rtsafe(x1,x2,xacc)
      Implicit Double Precision (A-z)
      Integer j, MAXIT
      Parameter (MAXIT=100)
      !
      Call funcd(x1,fl,df)
      Call funcd(x2,fh,df)
      !
      If ((fl.gt.0.D0.and.fh.gt.0.D0).or.(fl.lt.0.D0.and.fh.lt.0.D0))
     .   Pause 'root must be bracketed in rtsafe'
      !
      If (fl.eq.0.D0) Then
         rtsafe = x1
         Return
      Else If (fh.eq.0.D0) Then
         rtsafe = x2
         Return
      Else If (fl.lt.0.D0) Then
         xl = x1
         xh = x2
      Else
         xh = x1
         xl = x2
      End If
      !
      rtsafe = 0.5D0*(x1+x2)
      dxold = dabs(x2-x1)
      dx = dxold
      Call funcd(rtsafe,f,df)
      !
      Do 11 j = 1,MAXIT
         If (((rtsafe-xh)*df-f)*((rtsafe-xl)*df-f).gt.0.D0.or.dabs(2.D0*
     .   f).gt.dabs(dxold*df)) Then
            dxold = dx
            dx = 0.5D0*(xh-xl)
            rtsafe = xl+dx
            If (xl.eq.rtsafe) Return
         Else
            dxold = dx
            dx = f/df
            temp = rtsafe
            rtsafe = rtsafe-dx
            If (temp.eq.rtsafe) Return
         End If
         If (dabs(dx).lt.xacc) Return
         Call funcd(rtsafe,f,df)
         If (f.lt.0.D0) Then
            xl = rtsafe
         Else
            xh = rtsafe
         End If
11    Continue
      Pause 'rtsafe exceeding maximum iterations'
      Return
      End
C----------------------------------------------------------------------
      !
C     ! Given the lower and upper limits of integration x1 and x2, and
C     ! given n, this routine returns arrays x(1:n) and w(1:n) of length
C     ! n, containing the abcissas and weights of the Gauss-Legendre
C     ! n-point quadrature formula.
      !
      Subroutine gauleg(x1,x2,x,w,n)
      Implicit Double Precision (A-z)
      Integer i, j, m, n
      Dimension x(n), w(n)
      Parameter (Pi = 3.141592653589793238D0, eps = 3.D-14)
      !
C     ! The roots are symmetric in the interval,
C     ! so we only have to find half of them.
      !
      m = int(n+1)/2
      xm = 0.5D0*(x2+x1)
      xl = 0.5D0*(x2-x1)
      !
C     ! Loop over the desired roots.
      !
      Do 30 i=1,m
      !
C     ! Starting point to find the i-th root
      !
         z = dcos(Pi*(i-0.25D0)/(n+0.5D0))
      !
   10    p1 = 1.D0
         p2 = 0.D0
      !
C     ! Loop up the recurrence relation to get the
C     ! Legendre polynomial evaluated at z.
      !
         Do 20 j=1,n
            p3 = p2
            p2 = p1
   20       p1 = ((2.D0*j-1.D0)*z*p2-(j-1.D0)*p3)/j
      !
C     ! Calculate pp the derivative of the Legendre polynomial, by
C     ! a standard relation involving p1 and p2, the polynomial of
C     ! one lower order.
      !
         pp = n*(z*p1-p2)/(z*z-1.D0)
         z1 = z
      !
C     ! Newtons method.
      !
         z = z1-p1/pp
         If (dabs(z-z1).gt.eps) Goto 10
      !
C     ! Scale the root to the desired interval
C     ! and put in symmetric counterpart.
      !
         x(i) = xm-xl*z
         x(n+1-i) = xm+xl*z
      !
C     ! Compute the weight and its symmetric counterpart.
      !
         w(i) = 2.D0*xl/((1.D0-z*z)*pp*pp)
         w(n+1-i) = w(i)
   30 Continue
      !
      Return
      End
C-----------------------------------------------------------------------
