      !
C     ! Program for the CP exercise 4.1
C     ! Example that uses the cubic spline interpolation
C     ! from the Numerical Recipes
C     ! 
C     ! Last change: 2.06.2003
C     ! Matthias Jamin
      !
      Program Cubic_Spline_Interpolation
      Implicit Double Precision (A-z)
      Integer n, ndata
      !
      Parameter (ndata = 1185)
      Dimension SplineData(2,ndata)
      Dimension xint(ndata), yint(ndata), y2int(ndata)
      !
      !
C     ! Load data which shall be interpolated
      !
      Open (Unit=8, File='spline.dat', status='old')
      Read (8,*) SplineData
      Close (8)
      !
C     ! Provide vectors of x and y table values
      !
      Do 10 n=1,ndata
      xint(n) = SplineData(1,n)
   10 yint(n) = SplineData(2,n)
      !
C     ! Calculate 2nd derivatives
      !
      Call spline(xint,yint,y2int,ndata)
      !
      !
      Open (Unit=8, File='result.dat')
      !
      Do 20 n=1,ndata-1
      !
      x = 0.631D0 + n*0.002D0
      !
C     ! Calculate y = f(x) at x
C     ! This is already the improved subroutine with the
C     ! additional last argument n, to avoid the bisection.
      !
      Call splint(xint,yint,y2int,x,y,ndata,n)
   20 Write (8,*) x, y
      !
      Close (8)
      !
      Stop
      End
C----------------------------------------------------------------------
      !
C     ! Given arrays x(1:n) and y(1:n) containing a tabulated function
C     ! y_i = f(x_i), with x_1 < x_2 < ... < x_n, and given values yp1
C     ! and ypn for the first derivative of the interpolating function
C     ! at the points x_1 and x_n respectively, this routine returns an
C     ! array y2(1:n) of length n which contains the second derivatives
C     ! of the interpolating function at the tabulated points x_i. If
C     ! yp1 and/or ypn are equal to 10^30 or larger, the routine sets
C     ! the corresponding boundary condition to the natural spline with
C     ! zero second derivative on the boundary.
      !
      Subroutine spline(x,y,y2,n)
      Implicit Double Precision (A-z)
      Integer i, k, n
      Dimension x(n), y(n), y2(n), u(n)
      !
C     ! The lower boundary condition is either set to be natural,
      !
      If (yp1.gt.0.99D30) Then
         y2(1) = 0.D0
         u(1) = 0.D0
      !
C     ! or else to have the specified first derivative.
      !
      Else
         y2(1) = -0.5D0
         u(1) = (3.D0/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      End If
      !
C     ! This is the decomposition loop of the tridiagonal algorithm.
C     ! y2 and u are used for temporary storage of the decomposed factors.
      !
      Do 11 i=2,n-1
         sig = (x(i)-x(i-1))/(x(i+1)-x(i-1))
         p = sig*y2(i-1)+2.D0
         y2(i) = (sig-1.D0)/p
         u(i) = (6.D0*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))
     .          /(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
   11 Continue
      !
C     ! The upper boundary condition is either set to be natural,
      !
      If (ypn.gt.0.99D30) Then
         qn = 0.D0
         un = 0.D0
      Else
      !
C     ! or else to have the specified first derivative.
      !
         qn = 0.5D0
         un = (3.D0/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      End If
      !
      y2(n) = (un-qn*u(n-1))/(qn*y2(n-1)+1.D0)
      !
C     ! This is the backsubstitution loop of the tridiagonal algorithm
      !
      Do 12 k=n-1,1,-1
         y2(k) = y2(k)*y2(k+1)+u(k)
12    Continue
      !
      Return
      End
C-----------------------------------------------------------------------
      !
C     ! Given the arrays xa(1:n) and ya(1:n) of length n, which tabulate
C     ! a function (with ordered xa_i's), and given the array y2a(1:n),
C     ! which is the output from spline above, and given a value of x,
C     ! this routine returns a cubic-spline interpolation value y.
      !
      Subroutine splint(xa,ya,y2a,x,y,n,klo)
      Implicit Double Precision (A-z)
      Integer k, klo, khi, n
      Dimension xa(n), ya(n), y2a(n)
      !
C     ! This is the improved version which bypasses the bisection
      !
      khi = klo+1
      Goto 2
      !
C     ! This is the original version of the Numerical Recipes
      !
      klo = 1
      khi = n
      !
C     ! We shall find the right place in the table by means of bisection.
C     ! This is optimal if sequential calls to this routine are at random
C     ! values of x. If sequential calls are in order, and closely spaced,
C     ! one would do better to store previous values of klo and khi and
C     ! test if the remain appropriate on the next call.
      !
1     If (khi-klo.gt.1) Then
         k = (khi+klo)/2
         If (xa(k).gt.x) Then
            khi = k
         Else
            klo = k
         End If
      Goto 1
      End If
      !
C     ! klo and khi now bracket the input value of x
      !
2     h = xa(khi)-xa(klo)
      If (h.eq.0.D0) Pause 'Bad xa input.'
      a = (xa(khi)-x)/h
      b = (x-xa(klo))/h
      y = a*ya(klo)+b*ya(khi) +
     .    ((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.D0
      !
      Return
      End
C-----------------------------------------------------------------------
