(*** Solution for the exercise no 5.1, ICP, of June 6 ***)
(***              Linear Least Square Fit             ***)
(***             Matthias Jamin,  5.6.2003            ***)

(* Load the data set *)

sindat = ReadList[ "sinfit.dat", Number, RecordLists -> True];


(* The two sets of fit model functions *)

f[x_] := { 1, x, x^2, x^3};

g[x_] := { 1, x, x^3};


(* Normal equations in matrix form *)

(* Matrices A and al for model f(x) *)

Af = Table[ f[sindat[[i,1]]][[j]]/sindat[[i,3]], {i,21}, {j,4}];
AfT = Transpose[ Af];
alf = AfT.Af;

(* Matrices A and al for model g(x) *)

Ag = Table[ g[sindat[[i,1]]][[j]]/sindat[[i,3]], {i,21}, {j,3}];
AgT = Transpose[ Ag];
alg = AgT.Ag;

(* The vectors b and be *)

bb = Table[ sindat[[i,2]]/sindat[[i,3]], {i,21}];
bef = AfT.bb;
beg = AgT.bb;


(* Solutions in the 4- and 3-parameter cases *)

solf = LinearSolve[ alf, bef];
solg = LinearSolve[ alg, beg];


(* Error matrices, sigmas and correlation coefficients in both cases *)

Cf = Inverse[ alf];
Cg = Inverse[ alg];

sigf = Table[ Sqrt[ Cf[[i,i]]], {i,4}];
sigg = Table[ Sqrt[ Cg[[i,i]]], {i,3}];

rf = Table[ Cf[[i,j]]/sigf[[i]]/sigf[[j]], {i,4}, {j,4}];
rg = Table[ Cg[[i,j]]/sigg[[i]]/sigg[[j]], {i,3}, {j,3}];


(* The minimum chi^2 for both cases *)

chi2f = Sum[ ( (sindat[[n,2]] - Sum[ solf[[i]] f[sindat[[n,1]]][[i]], {i,4}])/
               sindat[[n,3]] )^2, {n,21}];

chi2g = Sum[ ( (sindat[[n,2]] - Sum[ solg[[i]] g[sindat[[n,1]]][[i]], {i,3}])/
               sindat[[n,3]] )^2, {n,21}];


(* Print out the results *)

Print[ ""];
Print[ "Solution for 4-parameter model: ", solf]; Print[ ""];
Print[ "Sigmas for 4-parameter model: ", sigf]; Print[ ""];
Print[ "Correlations for 4-parameter model:"];
Print[ ""]; Print[ MatrixForm[ rf]]; Print[ ""];
Print[ "Chi^2 for 4-parameter model: ", chi2f]; Print[ ""];

Print[ ""];
Print[ "Solution for 3-parameter model: ", solg]; Print[ ""];
Print[ "Sigmas for 3-parameter model: ", sigg]; Print[ ""];
Print[ "Correlations for 3-parameter model:"];
Print[ ""]; Print[ MatrixForm[ rg]]; Print[ ""];
Print[ "Chi^2 for 3-parameter model: ", chi2g]; Print[ ""];

