(*** Solution for the exercise no 5.2, ICP, of June 6 ***)
(***            Non-Linear Least Square Fit           ***)
(***             Matthias Jamin,  5.6.2003            ***)

(* Read the data set *)

lorentz = ReadList[ "lorentz.dat", Number, RecordLists -> True];


(* The fit model *)

f[la_] := b[4] + b[3]/(1+4((la-b[1])/b[2])^2);


(* The Chi^2 function *)

Chi2 = Sum[ ( (lorentz[[n,2]]-f[lorentz[[n,1]]])/lorentz[[n,3]])^2, {n,21}];
Chi2 = Chi2 // Expand;   (* Simpler form *)


(* Gradient vector and Hesse matrix *)

Grad  = Table[ D[ Chi2, b[i]], {i,4}];
Hesse = Table[ D[ Chi2, b[i], b[j]], {i,4}, {j,4}];


(* Initial trial solution *)

a[0] = { 4358.4, 0.3, 150.0, 25.0};


(* Perform Newton's algorithm starting from the trial solution *)

Do[

grad[k] = Grad /. Table[ b[i] -> a[k][[i]], {i,4}];
hesse[k] = Hesse /. Table[ b[i] -> a[k][[i]], {i,4}];

a[k+1] = a[k] - Inverse[ hesse[k]].grad[k];
chi2[k+1] = Chi2 /. Table[ b[i] -> a[k+1][[i]], {i,4}];
Print[ a[k+1], "   ", chi2[k+1]], {k,0,19}];

