      !
C     ! Exercise 7.2 of my CP lecture SS 2003
C     ! Gauss-Laguerre Integration
C     ! Last change: 24.6.2003
C     ! Matthias Jamin
      !
      Program GaussLaguerre
      Implicit Double Precision (A-z)
      Integer k, n, ngau, nmax
      !
      Parameter (nmax = 1024)
      Dimension xx(nmax), wx(nmax)
      !
      Data Pi /3.14159265358979323846264D0/
      !
C     ! Loop over the number of Gauss points
      !
      Do 200 k=1,6
      ngau = 2**k
      !
C     ! Initialise Gauss integration
      !
      Call gaulag(xx,wx,ngau,-0.5D0)
      !
      !
C     ! Calculate the integral
      !
      SUM = 0.D0
      !
      Do 100 n = 1,ngau
      !
  100 SUM = SUM + wx(n)
      !
      Int = SUM
  200 Write (*,*) k, Int, Int-dsqrt(Pi)
      !
      Stop
      End
C----------------------------------------------------------------------
      !
C     ! Given alf, the parameter of the Laguerre polynomial, this routine
C     ! returns arrays x(1:n) and w(1:n) of length n, containing the
C     ! abcissas and weights of the n-point Gauss-Laguerre quadrature
C     ! formula. The smallest abscissa is returned in x(1), the largest
C     ! in x(n).
      !
      Subroutine gaulag(x,w,n,alf)
      Implicit Double Precision (A-z) 
      Integer i, its, j, n, maxit
      Dimension x(n), w(n)
      Parameter (maxit = 30, eps = 3.D-14)
      !
C     USES gammln
      !
C     ! Loop over desired roots.
      !
      Do 30 i=1,n
         If (i.eq.1) Then
      !
C     ! Initial guess for smallest root.
      !
            z = (1.D0+alf)*(3.D0+0.92D0*alf)/(1.D0+2.4D0*n+1.8D0*alf)
         Else If (i.eq.2) Then
      !
C     ! Initial guess for second root.
      !
            z = z+(15.D0+6.25D0*alf)/(1.D0+0.9D0*alf+2.5D0*n)
         Else
      !
C     ! Initial guess for other roots.
      !
            ai = i-2
            z = z+((1.D0+2.55D0*ai)/(1.9D0*ai)+1.26D0*ai*alf/
     .          (1.D0+3.5D0*ai))*(z-x(i-2))/(1.D0+0.3D0*alf)
         End If
      !
C     ! Refinement by means of Newtons method.
      !
         Do 20 its=1,maxit
            p1 = 1.D0
            p2 = 0.D0
      !
C     ! Loop the recurrence relation to get the
C     ! Laguerre polynomial evaluated at z.
      !
            Do 10 j=1,n
               p3 = p2
               p2 = p1
               p1 = ((2*j-1+alf-z)*p2-(j-1+alf)*p3)/j
   10       Continue
      !
C     ! p1 is now the desired Laguerre polynomial. We next compute its
C     ! derivative pp, by a standard relation involving also p2, the
C     ! polynomial of one lower order.
      !
            pp = (n*p1-(n+alf)*p2)/z
            z1 = z
      !
C     ! Newtons method.
      !
            z = z1-p1/pp
            If (dabs(z-z1).le.eps) Goto 1
   20    Continue
         Pause 'too many iterations in gaulag'
      !
C     ! Store the roots and weights.
      !
    1    x(i) = z
         w(i) = -dexp(gammln(alf+n)-gammln(dble(n)))/(pp*n*p2)
   30 Continue
      !
      Return
      End
C-----------------------------------------------------------------------
      !
C     ! Returns the value log(gamma(xx)) for xx > 0.
      !
      Double Precision Function gammln(xx)
      Implicit Double Precision (A-z) 
      Integer j
      Dimension cof(6)
      Save cof, stp
      Data cof, stp /76.18009172947146D0,   -86.50532032941677D0,
     .               24.01409824083091D0,    -1.231739572450155D0,
     .                0.1208650973866179D-2, -0.5395239384953D-5,
     .                2.5066282746310005D0/
      !
      x = xx
      y = x
      tmp = x+5.5D0
      tmp = (x+0.5D0)*dlog(tmp)-tmp
      ser =1.000000000190015D0
      Do 10 j=1,6
         y = y+1.d0
         ser = ser+cof(j)/y
   10 Continue
      gammln = tmp+dlog(stp*ser/x)
      !
      Return
      End
C-----------------------------------------------------------------------
