      !
C     ! Exercise 7.4 of my CP lecture SS 2003
C     ! Gauss-Legendre Integration of Data
C     ! Last change: 24.6.2003
C     ! Matthias Jamin
      !
      Program GaussLegendre
      Implicit Double Precision (A-z)
      Integer k, n, ndata, ngau, nmax
      !
      Parameter (ndata = 1185, nmax = 4096)
      Dimension Data(2,ndata), xx(nmax), wx(nmax)
      Dimension xint(ndata), yint(ndata), y2int(ndata)
      Dimension Int(12)
      !
      Data Pi /3.14159265358979323846264D0/
      !
C     ! Load data set
      !
      Open (Unit=8, File='spline.dat', status='old')
      Read (8,*) Data
      Close (8)
      !
C     ! Provide vectors of x and y table values
      !
      Do 10 n=1,ndata
      xint(n) = Data(1,n)
   10 yint(n) = Data(2,n)
      !
      !
C     ! Calculate integral with trapezoidal rule and half the data
      !
      h = 4.D-3
      SUM = 0.5D0*h*(yint(1)+yint(ndata))
      !
      Do 20 n = 1,592
   20 SUM = SUM + h*yint(2*n+1)
      !
      Int(1) = SUM
      Write (*,*) 1, Int(1)
      !
      !
C     ! Calculate integral with trapezoidal rule and full data set
      !
      h = 2.D-3
      SUM = 0.5D0*h*(yint(1)+yint(ndata))
      !
      Do 30 n = 2,1184
   30 SUM = SUM + h*yint(n)
      !
      Int(2) = SUM
      Write (*,*) 2, Int(2), Int(2)-Int(1)
      !
      !
C     ! Result of first Romberg step
      !
      Int(3) = (4.D0*Int(2)-Int(1))/3.D0
      Write (*,*) 3, Int(3), Int(3)-Int(2)
      !
      !
C     ! Prepare spline interpolation by calculating 2nd derivatives
      !
      Call spline(xint,yint,y2int,ndata)
      !
C     ! Loop over the number of Gauss points
      !
      Do 200 k=4,12
      ngau = 2**k
      !
C     ! Initialise Gauss integration
      !
      Call gauleg(xint(1),xint(ndata),xx,wx,ngau)
      !
      !
C     ! Calculate the integral
      !
      SUM = 0.D0
      !
      Do 100 n = 1,ngau
      !
      Call splint(xint,yint,y2int,xx(n),ydata,ndata)
  100 SUM = SUM + wx(n)*ydata
      !
      Int(k) = SUM
  200 Write (*,*) k, Int(k), Int(k)-Int(k-1), Int(k)-Int(3)
      !
      Stop
      End
C----------------------------------------------------------------------
      !
C     ! Given the lower and upper limits of integration x1 and x2, and
C     ! given n, this routine returns arrays x(1:n) and w(1:n) of length
C     ! n, containing the abcissas and weights of the Gauss-Legendre
C     ! n-point quadrature formula.
      !
      Subroutine gauleg(x1,x2,x,w,n)
      Implicit Double Precision (A-z)
      Integer i, j, m, n
      Dimension x(n), w(n)
      Parameter (Pi = 3.141592653589793238D0, eps = 3.D-14)
      !
C     ! The roots are symmetric in the interval,
C     ! so we only have to find half of them.
      !
      m = int(n+1)/2
      xm = 0.5D0*(x2+x1)
      xl = 0.5D0*(x2-x1)
      !
C     ! Loop over the desired roots.
      !
      Do 30 i=1,m
      !
C     ! Starting point to find the i-th root
      !
         z = dcos(Pi*(i-0.25D0)/(n+0.5D0))
      !
   10    p1 = 1.D0
         p2 = 0.D0
      !
C     ! Loop up the recurrence relation to get the
C     ! Legendre polynomial evaluated at z.
      !
         Do 20 j=1,n
            p3 = p2
            p2 = p1
   20       p1 = ((2.D0*j-1.D0)*z*p2-(j-1.D0)*p3)/j
      !
C     ! Calculate pp the derivative of the Legendre polynomial, by
C     ! a standard relation involving p1 and p2, the polynomial of
C     ! one lower order.
      !
         pp = n*(z*p1-p2)/(z*z-1.D0)
         z1 = z
      !
C     ! Newtons method.
      !
         z = z1-p1/pp
         If (dabs(z-z1).gt.eps) Goto 10
      !
C     ! Scale the root to the desired interval
C     ! and put in symmetric counterpart.
      !
         x(i) = xm-xl*z
         x(n+1-i) = xm+xl*z
      !
C     ! Compute the weight and its symmetric counterpart.
      !
         w(i) = 2.D0*xl/((1.D0-z*z)*pp*pp)
         w(n+1-i) = w(i)
   30 Continue
      !
      Return
      End
C-----------------------------------------------------------------------
      !
C     ! Given arrays x(1:n) and y(1:n) containing a tabulated function
C     ! y_i = f(x_i), with x_1 < x_2 < ... < x_n, and given values yp1
C     ! and ypn for the first derivative of the interpolating function
C     ! at the points x_1 and x_n respectively, this routine returns an
C     ! array y2(1:n) of length n which contains the second derivatives
C     ! of the interpolating function at the tabulated points x_i. If
C     ! yp1 and/or ypn are equal to 10^30 or larger, the routine sets
C     ! the corresponding boundary condition to the natural spline with
C     ! zero second derivative on the boundary.
      !
      Subroutine spline(x,y,y2,n)
      Implicit Double Precision (A-z)
      Integer i, k, n
      Dimension x(n), y(n), y2(n), u(n)
      !
C     ! The lower boundary condition is either set to be natural,
      !
      If (yp1.gt.0.99D30) Then
         y2(1) = 0.D0
         u(1) = 0.D0
      !
C     ! or else to have the specified first derivative.
      !
      Else
         y2(1) = -0.5D0
         u(1) = (3.D0/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      End If
      !
C     ! This is the decomposition loop of the tridiagonal algorithm.
C     ! y2 and u are used for temporary storage of the decomposed factors.
      !
      Do 11 i=2,n-1
         sig = (x(i)-x(i-1))/(x(i+1)-x(i-1))
         p = sig*y2(i-1)+2.D0
         y2(i) = (sig-1.D0)/p
         u(i) = (6.D0*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))
     .          /(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
   11 Continue
      !
C     ! The upper boundary condition is either set to be natural,
      !
      If (ypn.gt.0.99D30) Then
         qn = 0.D0
         un = 0.D0
      Else
      !
C     ! or else to have the specified first derivative.
      !
         qn = 0.5D0
         un = (3.D0/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      End If
      !
      y2(n) = (un-qn*u(n-1))/(qn*y2(n-1)+1.D0)
      !
C     ! This is the backsubstitution loop of the tridiagonal algorithm
      !
      Do 12 k=n-1,1,-1
         y2(k) = y2(k)*y2(k+1)+u(k)
12    Continue
      !
      Return
      End
C-----------------------------------------------------------------------
      !
C     ! Given the arrays xa(1:n) and ya(1:n) of length n, which tabulate
C     ! a function (with ordered xa_i's), and given the array y2a(1:n),
C     ! which is the output from spline above, and given a value of x,
C     ! this routine returns a cubic-spline interpolation value y.
      !
      Subroutine splint(xa,ya,y2a,x,y,n)
      Implicit Double Precision (A-z)
      Integer k, klo, khi, n
      Dimension xa(n), ya(n), y2a(n)
      !
      klo = 1
      khi = n
      !
C     ! We shall find the right place in the table by means of bisection.
C     ! This is optimal if sequential calls to this routine are at random
C     ! values of x. If sequential calls are in order, and closely spaced,
C     ! one would do better to store previous values of klo and khi and
C     ! test if the remain appropriate on the next call.
      !
1     If (khi-klo.gt.1) Then
         k = (khi+klo)/2
         If (xa(k).gt.x) Then
            khi = k
         Else
            klo = k
         End If
      Goto 1
      End If
      !
C     ! klo and khi now bracket the input value of x
      !
      h = xa(khi)-xa(klo)
      If (h.eq.0.D0) Pause 'Bad xa input.'
      a = (xa(khi)-x)/h
      b = (x-xa(klo))/h
      y = a*ya(klo)+b*ya(khi) +
     .    ((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.D0
      !
      Return
      End
C-----------------------------------------------------------------------
