(*** Solution for the exercise no 8.1, ICP, of July 4 ***)
(***               Euler Methods for DGL              ***)
(***             Matthias Jamin, 30.6.2003            ***)

(* Define the derivative from the DGL: y' = y^2 + 1 *)

yp[x_,y_] := y^2 + 1;

(* Already includes an x-dependence of yp[x,y]  *)
(* for future use, although in the present      *)
(* exercise such a dependence does not exist.   *)


(* Implementation of the simple Euler Algorithm *)

SEuler[y0_,a_,b_,h_] := Module[ {nmax},
                        y[0] = y0; nmax = Round[(b-a)/h];
                        Do[ y[n+1] = y[n] + h yp[n h,y[n]], {n,0,nmax-1}];
                        Table[ {n h, y[n]}, {n,0,nmax}]];

SolS10 = SEuler[0,0,1,0.10];
SolS05 = SEuler[0,0,1,0.05];


(* Implementation of the modified Euler Algorithm *)

MEuler[y0_,a_,b_,h_] := Module[ {nmax},
                        y[0] = y0; nmax = Round[(b-a)/h];
       Do[ y[n+1] = y[n] + h yp[(n+1/2)h,y[n]+h/2 yp[n h,y[n]]], {n,0,nmax-1}];
                        Table[ {n h, y[n]}, {n,0,nmax}]];

SolM10 = MEuler[0,0,1,0.10];
SolM05 = MEuler[0,0,1,0.05];


(* Implementation of the improved Euler Algorithm *)

IEuler[y0_,a_,b_,h_] := Module[ {nmax},
                        y[0] = y0; nmax = Round[(b-a)/h];
       Do[ y[n+1] = y[n] + h/2 (yp[n h,y[n]] +
                    yp[(n+1)h,y[n]+h yp[n h,y[n]]]), {n,0,nmax-1}];
                        Table[ {n h, y[n]}, {n,0,nmax}]];

SolI10 = IEuler[0,0,1,0.10];
SolI05 = IEuler[0,0,1,0.05];


(* Exact result Tan[x] with step-sizes 0.10 and 0.05 for comparison *)

SolE10 = Table[ { x, Tan[x]}, {x,0,1,0.10}];
SolE05 = Table[ { x, Tan[x]}, {x,0,1,0.05}];


(* Function to give the relative difference of two solutions *)
(* The solutions must have the same number of points!        *)

Compare[sol1_,sol2_] := Module[ { n1=Length[sol1], n2=Length[sol2]},
        If[ n1==n2,
        Table[ {sol1[[n,1]],(sol1[[n,2]]-sol2[[n,2]])/sol1[[n,2]]}, {n,2,n1}],
        Print[ "The two solutions have different numbers of points!"]]];


(*  Examples of comparisons   *)
(* Just enter Com10 or Com05! *)

ComE10S10 := ListPlot[ Compare[ SolE10, SolS10]];
ComE10M10 := ListPlot[ Compare[ SolE10, SolM10]];
ComE10I10 := ListPlot[ Compare[ SolE10, SolI10]];

Com10 := Show[ ComE10S10, ComE10M10, ComE10I10];


ComE05S05 := ListPlot[ Compare[ SolE05, SolS05]];
ComE05M05 := ListPlot[ Compare[ SolE05, SolM05]];
ComE05I05 := ListPlot[ Compare[ SolE05, SolI05]];

Com05 := Show[ ComE05S05, ComE05M05, ComE05I05];

