(*** Solution for the exercise no 8.2, ICP, of July 4 ***)
(***               Conserved Quantities               ***)
(***             Matthias Jamin, 30.6.2003            ***)


(* Implementation of the simple Euler Algorithm for this exercise *)

SOszi[h_] := Module[ {nmax},
                     x[0] = 0;  v[0] = 1;  nmax = Round[6/h];
                     Do[ v[n+1] = v[n] - h x[n];
                         x[n+1] = x[n] + h v[n], {n,0,nmax-1}];
                     Do[ e[n] = (v[n]^2 + x[n]^2)/2, {n,0,nmax}];   
  Transpose[ Table[ {{n h, v[n]}, {n h, x[n]}, {n h, e[n]}}, {n,0,nmax}]]];

SolS = SOszi[0.1]; SolSv = SolS[[1]]; SolSx = SolS[[2]]; SolSe = SolS[[3]];


(* Implementation of the modified Euler Algorithm for this exercise *)

MOszi[h_] := Module[ {nmax},
                     x[0] = 0;  v[0] = 1;  nmax = Round[6/h];
                     Do[ v[n+1/2] = v[n] - h/2 x[n];
                         x[n+1/2] = x[n] + h/2 v[n];
                         v[n+1] = v[n] - h x[n+1/2];
                         x[n+1] = x[n] + h v[n+1/2], {n,0,nmax-1}];
                     Do[ e[n] = (v[n]^2 + x[n]^2)/2, {n,0,nmax}];
  Transpose[ Table[ {{n h, v[n]}, {n h, x[n]}, {n h, e[n]}}, {n,0,nmax}]]];

SolM = MOszi[0.1]; SolMv = SolM[[1]]; SolMx = SolM[[2]]; SolMe = SolM[[3]];


(* Implementation of the simple Euler Algorithm using the energy E *)

SEOszi[h_] := Module[ {nmax},
                      x[0] = 0;  v[0] = 1;  nmax = Round[6/h];
                      Do[ x[n+1] = x[n] + h v[n];
                          v[n+1] = Sqrt[1-x[n+1]^2] Sign[ v[n]-h x[n]],
                          {n,0,nmax-1}];
           Transpose[ Table[ {{n h, v[n]}, {n h, x[n]}}, {n,0,nmax}]]];

SolSE = SEOszi[0.1];  SolSEv = SolSE[[1]];  SolSEx = SolSE[[2]];


(* Implementation of the modified Euler Algorithm using the energy E *)

MEOszi[h_] := Module[ {nmax},
                      x[0] = 0;  v[0] = 1;  nmax = Round[6/h];
                      Do[ v[n+1/2] = v[n] - h/2 x[n];
                          x[n+1/2] = x[n] + h/2 v[n];
                          v[n+1] = v[n] - h x[n+1/2];
                          x[n+1] = Sqrt[1-v[n+1]^2] Sign[ x[n]+h v[n+1/2]],
                          {n,0,nmax-1}];
           Transpose[ Table[ {{n h, v[n]}, {n h, x[n]}}, {n,0,nmax}]]];

SolME = MEOszi[0.1];  SolMEv = SolME[[1]];  SolMEx = SolME[[2]];


(* Exact solutions to the differential equation *)

SolEv = Table[ { x, Cos[x]}, {x,0.,6.,0.1}];
SolEx = Table[ { x, Sin[x]}, {x,0.,6.,0.1}];


(* Function to give the relative difference of two solutions *)
(* The solutions must have the same number of points!        *)

Compare[sol1_,sol2_] := Module[ { n1=Length[sol1], n2=Length[sol2]},
        If[ n1==n2,
        Table[ {sol1[[n,1]],(sol1[[n,2]]-sol2[[n,2]])/sol1[[n,2]]}, {n,2,n1}],
        Print[ "The two solutions have different numbers of points!"]]];


ComESx := ListPlot[ Compare[ SolEx, SolSx]];
ComEMx := ListPlot[ Compare[ SolEx, SolMx]];
ComESMx := Show[ ComESx, ComEMx];

ComESv := ListPlot[ Compare[ SolEv, SolSv]];
ComEMv := ListPlot[ Compare[ SolEv, SolMv]];
ComESMv := Show[ ComESv, ComEMv];

