(*** Solution for the exercise no 8.3, ICP, of July 4 ***)
(***               Runge-Kutta Algorithm              ***)
(***             Matthias Jamin, 30.6.2003            ***)

(* Numerical Constants *)

g = 9.80665;   m = 0.01;   k = 0.0001;


(* Define the derivative from the DGL: dv/dt = g - k/m v^2 *)

vp[t_,v_] := g - k/m v^2;

(* Already includes a t-dependence of vp[t,v] *)
(* for future use, although in the present    *)
(* exercise such a dependence does not exist. *)

(* Exact solution *)

ve[t_] := Sqrt[m g/k] Tanh[ Sqrt[g k/m] t];


(* Implementation of the Runge-Kutta Algorithm with adaptive step size *)

RKA[ta_,tf_,va_,ha_,eps_] := Module[ {t0=ta,v0=va,v1,v2,vr,h=ha,sol={{ta,va}}},
                            While[ t0 <= tf,
                                   v1 = RKStep[t0,v0,h/2];
                                   v2 = RKStep[t0+h/2,v1,h/2];
                                   v1 = RKStep[t0,v0,h];
                                   vr = (16v2-v1)/15;
                                   If[ Abs[vr-v2]<eps, t0 = t0+h; v0 = vr;
                                       sol = Union[ sol, {{t0,vr}}], h = h/2]];
                                     sol];

RKStep[t0_,v0_,h_] := Module[ { f0, f1, f2, f3},
                              f0 = vp[ t0, v0];
                              f1 = vp[ t0+h/2, v0+h/2 f0];
                              f2 = vp[ t0+h/2, v0+h/2 f1];
                              f3 = vp[ t0+h, v0+h f2];
                              v0 + h/6 (f0+2f1+2f2+f3)];


SolRK = RKA[0,10,0,2.,2 10^-5];
SolEx = Table[ { t, ve[t]}, {t,0,10.5,0.5}];
Solk0 = Table[ { t, g t}, {t,0,10.5,0.5}];


(* Display the solutions *)

PlotSolRK := ListPlot[ SolRK];
PlotSolEx := ListPlot[ SolEx];
PlotSolk0 := ListPlot[ Solk0];

PlotSol := Show[ PlotSolRK, PlotSolEx, PlotSolk0];


(* Function to give the relative difference of two solutions *)
(* The solutions must have the same number of points!        *)

Compare[sol1_,sol2_] := Module[ { n1=Length[sol1], n2=Length[sol2]},
        If[ n1==n2,
        Table[ {sol1[[n,1]],(sol1[[n,2]]-sol2[[n,2]])/sol1[[n,2]]}, {n,2,n1}],
        Print[ "The two solutions have different numbers of points!"]]];

Com := Compare[ SolEx, SolRK];

