      !
C     ! Exercise 9.2 of my CP lecture SS 2003
C     ! Solution of the boundary condition problem
C     ! with the Cash-Karp Runge-Kutta method
C     ! Last change: 10.7.2003
C     ! Matthias Jamin
      !
      Program CashKarpRungeKutta
      Implicit Double Precision (A-z)
      Integer kmax, kount, nok, nbad, nvar
      !
      Parameter (nvar=2, NMAX=50, KMAXX=1000, eps=1.D-12)
      Dimension ystart(nvar), xp(KMAXX), yp(NMAX,KMAXX)
      Common /path/ kmax, kount, dxsav, xp, yp
      !
      kmax = 169
      !
      xstart = 0.D0
      ystart(1) = 0.0D0
      !
C     ! Something like Newtons algorithm still needs to be
C     ! implemented, in order to obtain the following result.
      !
      ystart(2) = 16.1451356618789D0
      !
      !
      Open (Unit=8, File='ex9.2.out')
      Write (8,FMT='(2F16.9)') xstart, ystart(1)
      !
      Call odeint(ystart,nvar,0.D0,1.D0,eps,0.2D0,0.D0,nok,nbad)
      !
      Write (*,*) nok, nbad
      Write (*,*) ystart(1)-1.D2, ystart(2)
      !
      Do 10 i = 1,kmax
   10 Write (8,FMT='(2F16.9)') xp(i), yp(1,i)
      !
      Close (8)
      !
      Stop
      End
C----------------------------------------------------------------------
      !
C     ! Calculates the right-hand side derivatives
C     ! for the DGL: y'' - 5 y' + 10 y(x) = 10 x
      !
      Subroutine derivs(x,y,dxdy)
      Implicit Double Precision (A-z)
      !
      Parameter (nvar=2)
      Dimension y(nvar), dxdy(nvar)
      !
      dxdy(1) = y(2)
      dxdy(2) = 5.D0*y(2)+1.D1*(x-y(1))
      !
      Return
      End
C----------------------------------------------------------------------
      !
C     ! Runge-Kutta driver with adaptive stepsize control. Integrate the
C     ! starting values ystart(1:nvar) from x1 to x2 with accuracy eps,
C     ! storing intermediate results in the common block /path/. h1 should
C     ! be set as a guessed first stepsize, hmin as the minimum allowed
C     ! stepsize (can be zero). On output, nok and nbad are the number of
C     ! good and bad (but retried and fixed) steps taken, and ystart is
C     ! replaced by values at the end of the integration interval. derivs
C     ! is the user-supplied subroutine for calculating the right-hand
C     ! side derivative, while rkqs is the name of the stepper routine to
C     ! be used. /path/ contains its own information about how often an
C     ! intermediate value is to be stored.
      !
      Subroutine odeint(ystart,nvar,x1,x2,eps,h1,hmin,nok,nbad)
      Implicit Double Precision (A-z)
      Integer i, kmax, kount, nbad, nok, nvar, nstp, KMAXX, MAXSTP, NMAX
      Dimension ystart(nvar)
C     External derivs, rkqs
      Parameter (MAXSTP=10000, NMAX=50, KMAXX=1000, TINY=1.D-30)
      Dimension dydx(NMAX),xp(KMAXX),y(NMAX),yp(NMAX,KMAXX),yscal(NMAX)
      Common /path/ kmax, kount, dxsav, xp, yp
      !
      x = x1
      h = dsign(h1,x2-x1)
      nok = 0
      nbad = 0
      kount = 0
      !
      Do 11 i = 1,nvar
        y(i) = ystart(i)
   11 Continue
      !
C     ! Assures storage of first step. 
      !
      If (kmax.gt.0) xsav = x-2.D0*dxsav
      !
C     ! Take at most MAXSTP steps.
      !
      Do 16 nstp = 1,MAXSTP
         Call derivs(x,y,dydx)
      !
C     ! Scaling used to monitor accuracy.
C     ! This general-purpose choice can be modified if need be.
      !
         Do 12 i = 1,nvar
            yscal(i) = dabs(y(i))+dabs(h*dydx(i))+TINY
   12    Continue
         If (kmax.gt.0) Then
            If (dabs(x-xsav).gt.dabs(dxsav)) Then
               If (kount.lt.kmax-1) Then
                  kount = kount+1
                  xp(kount) = x
                  Do 13 i = 1,nvar
                     yp(i,kount) = y(i)
   13             Continue
                  xsav = x
               End If
            End If
         End If
      !
C     ! If stepsize can overshoot, decrease.
      !
         If ((x+h-x2)*(x+h-x1).gt.0.D0) h = x2-x
         Call rkqs(y,dydx,nvar,x,h,eps,yscal,hdid,hnext)
         If (hdid.eq.h) Then
            nok = nok+1
         Else
            nbad = nbad+1
         End If
      !
C     ! Are we done?
      !
         If ((x-x2)*(x2-x1).ge.0.D0) Then
            Do 14 i = 1,nvar
               ystart(i) = y(i)
   14       Continue
      !
C     ! Save final step
      !
            If (kmax.ne.0) Then
               kount = kount+1
               xp(kount) = x
               Do 15 i = 1,nvar
                  yp(i,kount) = y(i)
   15          Continue
            End If
      !
C     ! Normal exit
      !
            Return
         End If
         If (dabs(hnext).lt.hmin) Pause
     . 'stepsize smaller than minimum in odeint'
         h = hnext
   16 Continue
      Pause 'too many steps in odeint'
      Return
      End
C----------------------------------------------------------------------
      !
C     ! Fifth order Runge-Kutta step with monitoring of local truncation
C     ! error to ensure accuracy and adjust stepsize. Input are the
C     ! dependent variable vector y(1:n) and its derivative dydx(1:n) at
C     ! the starting values of the independent variable x. Also input are
C     ! the stepsize to be attempted htry, the required accuracy eps, and
C     ! the vector yscal(1:n) against which the error is scaled. On output,
C     ! y and x are replaced by their new values, hdid is the stepsize
C     ! that was actually accomplished, and hnext is the estimated next
C     ! stepsize. derivs is the user-supplied subroutine that computes
C     ! the right-hand side derivatives.
      !
      Subroutine rkqs(y,dydx,n,x,htry,eps,yscal,hdid,hnext)
      Implicit Double Precision (A-z)
      INTEGER i, n, NMAX
      Dimension dydx(n), y(n), yscal(n)
C     External derivs
      !
C     ! Maximum number of equations
      !
      Parameter (NMAX=50)
      Dimension yerr(NMAX), ytemp(NMAX)
      Parameter (SAFETY=.9D0,PGROW=-.2D0,PSHRNK=-.25D0,ERRCON=1.89D-4)
      !
C     ! Set stepsize to initial trial value and take the step
      !
      h = htry
    1 Call rkck(y,dydx,n,x,h,ytemp,yerr)
      !
C     ! Evaluate accuracy
      !
      errmax = 0.D0
      Do 11 i = 1,n
         errmax = max(errmax,dabs(yerr(i)/yscal(i)))
   11 Continue
      !
C     ! Scale relative to required tolerance
      !
      errmax = errmax/eps
      If (errmax.gt.1.D0) Then
      !
C     ! Truncation error to large, reduce stepsize.
      !
         htemp = SAFETY*h*(errmax**PSHRNK)
         h = dsign(max(dabs(htemp),0.1D0*dabs(h)),h)
         xnew = x+h
         If (xnew.eq.x) pause 'stepsize underflow in rkqs'
      !
C     ! For another try
      !
         Goto 1
      !
C     ! Step succeeded. Compute size of next step
      !
      Else
         If (errmax.gt.ERRCON) Then
            hnext = SAFETY*h*(errmax**PGROW)
         Else
C     !     No more than a factor of 5 increase.
            hnext = 5.D0*h
         End If
         hdid = h
         x = x+h
         Do 12 i = 1,n
            y(i) = ytemp(i)
   12    Continue
         Return
      !
      End If
      End
C----------------------------------------------------------------------
C     ! Given values for n variables y and their derivatives dydx known
C     ! at x, use the fifth order Cash-Karp Runge-Kutta method to advance
C     ! the solution over an interval h and return the incremental
C     ! variables as yout. Also return an estimate of the local truncation
C     ! error in yout using the embedded fourth order method. The user
C     ! supplies the subroutine derivs(x,y,dydx), which returns the
C     ! derivatives dydx at x.
      !
      Subroutine rkck(y,dydx,n,x,h,yout,yerr)
      Implicit Double Precision (A-z)
      Integer i, n, NMAX
      Dimension dydx(n), y(n), yerr(n), yout(n)
C     External derivs
      !
C     ! Maximum number of equations
      !
      Parameter (NMAX=50)
      Dimension ak2(NMAX), ak3(NMAX), ak4(NMAX), ak5(NMAX), ak6(NMAX),
     .          ytemp(NMAX)
      Parameter (A2=0.2D0, A3=0.3D0, A4=0.6D0, A5=1.0D0, A6=0.875D0,
     . B21=0.2D0, B31=3.D0/40.D0, B32=9.D0/40.D0, B41=0.3D0,
     . B42=-0.9D0, B43=1.2D0, B51=-11.D0/54.D0, B52=2.5D0,
     . B53=-70.D0/27.D0, B54=35.D0/27.D0, B61=1631.D0/55296.D0,
     . B62=175.D0/512.D0, B63=575.D0/13824.D0, B64=44275.D0/110592.D0,
     . B65=253.D0/4096.D0, C1=37.D0/378.D0, C3=250.D0/621.D0,
     . C4=125.D0/594.D0, C6=512.D0/1771.D0, DC1=C1-2825.D0/27648.D0,
     . DC3=C3-18575.D0/48384.D0, DC4=C4-13525.D0/55296.D0,
     . DC5=-277.D0/14336.D0, DC6=C6-0.25D0)
      !
      Do 11 i = 1,n
         ytemp(i) = y(i)+B21*h*dydx(i)
   11 Continue
      Call derivs(x+A2*h,ytemp,ak2)
      !
      Do 12 i = 1,n
         ytemp(i) = y(i)+h*(B31*dydx(i)+B32*ak2(i))
   12 Continue
      Call derivs(x+A3*h,ytemp,ak3)
      !
      Do 13 i = 1,n
         ytemp(i) = y(i)+h*(B41*dydx(i)+B42*ak2(i)+B43*ak3(i))
   13 Continue
      Call derivs(x+A4*h,ytemp,ak4)
      !
      Do 14 i = 1,n
         ytemp(i)=y(i)+h*(B51*dydx(i)+B52*ak2(i)+B53*ak3(i)+B54*ak4(i))
   14 Continue
      Call derivs(x+A5*h,ytemp,ak5)
      !
      Do 15 i = 1,n
         ytemp(i) = y(i)+h*(B61*dydx(i)+B62*ak2(i)+B63*ak3(i)+
     .              B64*ak4(i)+B65*ak5(i))
   15 Continue
      !
      Call derivs(x+A6*h,ytemp,ak6)
      Do 16 i = 1,n
         yout(i) = y(i)+h*(C1*dydx(i)+C3*ak3(i)+C4*ak4(i)+C6*ak6(i))
   16 Continue
      !
      Do 17 i = 1,n
         yerr(i) = h*(DC1*dydx(i)+DC3*ak3(i)+DC4*ak4(i)+DC5*ak5(i)+
     .             DC6*ak6(i))
   17 Continue
      !
      Return
      End
C----------------------------------------------------------------------
