C----------------------------------------------------------------------
      !
C     ! Small program to test the subroutine "cauleg"
C     ! Requires the CERN library "mathlib" to run
C     ! Last change: 12.1.2001
C     ! Matthias Jamin
      !
      Program CauLegTest
      Implicit Double Precision (A-z)
      Integer i, j, ngau
      Parameter (ngau = 63)
      Dimension x(ngau+1), y(ngau), w(ngau+1,ngau)
      Common /CauInt/ z
      External fint
      !
      a = 0.D0
      b = 3.D0
      Call cauleg(a,b,x,y,w,ngau)
      !
      !
      Open (unit=8, file='ff.out')
      !
      Do 20 i=1,ngau+1
      !
      sum = 0.D0
      Do 10 j=1,ngau
   10 sum = sum + dsin(y(j))**2*w(i,j)
      si1 = sum
      !
C     ! Exact result
      !
      z = x(i)
      ex = 0.5D0*( dcos(2.D0*z)*(dcosin(2.D0*(a-z))-dcosin(2.D0*(b-z)))
     .           + dsin(2.D0*z)*(dsinin(2.D0*(z-a))-dsinin(2.D0*(z-b)))
     .           + dlog((b-z)/(z-a)) )
      !
   20 Write (8,*) z, si1, ex, dlog(dabs((si1-ex)/ex))/dlog(1.D1)
      !
      Close (8)
      !
      Return
      End
C-----------------------------------------------------------------------
      Double Precision Function fint(y)
      Implicit Double Precision (A-z)
      Common /CauInt/ z
      !
      fint = dsin(y)**2/(z-y)
      !
      Return
      End
C-----------------------------------------------------------------------
      !
C     ! Gauss routine for integration with the Cauchy kernel 1/(x-y)
C     ! based on the routine "gauleg" from the Numerical Recipes.
C     ! Uses legp(x,n) and legq(x,n)
      !
C     ! Given the lower and upper limits of integration x1 and x2, and
C     ! given n, the routine returns arrays x(1:n+1), y(1:n), w(1:n+1,1:n)
C     ! containing the points x where the integral should be evaluated,
C     ! as well as the abscissas and the weights of the Gauss-Legendre
C     ! n-point quadrature formula. x(1:n+1) and y(1:n) contain the roots
C     ! of the Legendre functions Q_n and P_n of order n, shifted to the
C     ! interval (x1,x2). Other values of x have to be calculated with
C     ! standard interpolation methods.
      !
      Subroutine cauleg(x1,x2,x,y,w,n)
      Implicit Double Precision (A-z)
      Integer i, j, m, n
      Dimension x(n+1), y(n), w(n+1,n)
      Parameter (Pi = 3.141592653589793238D0, eps = 1.D-15)
      !
      m = (n+1)/2
      xm = 0.5D0*(x2+x1)
      xl = 0.5D0*(x2-x1)
      !
      Do 20 i=1,m
         z = dcos(Pi*(i-0.25D0)/(n+0.5D0))
   10    z1 = z
         Call legp(p1,pp,z,n)
         z = z1-p1/pp
         If (dabs(z-z1).gt.eps) Goto 10
         y(i) =     xm-xl*z
         y(n-i+1) = xm+xl*z
   20 Continue
      !
      Do 40 i=1,m+1
         If (i.eq.1) Then
            z = -1.D0+1.D-9
         Else
            z = 0.5D0*(y(i)+y(i-1)-2.D0*xm)/xl
         End If
   30    z1 = z
         Call legq(q1,qp,z,n)
         z = z1-q1/qp
         If (dabs(z-z1).gt.eps) Goto 30
         x(i) =     xm+xl*z
         x(n-i+2) = xm-xl*z
   40 Continue
      !
      Do 50 i=1,m+1
         xi = (x(i)-xm)/xl
         Call legp(p1,pp,xi,n)
         Call legq(q2,qp,xi,n-1)
         Do 50 j=1,n
            yj = (y(j)-xm)/xl
            Call legp(p2,pp,yj,n-1)
            w(i,j) = 2.D0/dble(n)*(1.D0-yj*yj)
     .               /(yj-xi)*p1*q2/p2/p2
            w(n-i+2,n-j+1) = -w(i,j)
   50 Continue
      !
      Return
      End
C-----------------------------------------------------------------------
      !
C     ! Calculates the Legendre polynomials P_n(x) and P'_n(x)
C     ! P_n(x) is in p1 and P'_n(x) in pp.
      !
      Subroutine legp(p1,pp,x,n)
      Implicit Double Precision (A-z)
      Integer j, n
      !
      p2 = 1.D0
      p1 = x
      Do 10 j=2,n
         p3 = p2
         p2 = p1
         p1 = ((2.D0*j-1.D0)*x*p2-(j-1.D0)*p3)/j
   10 Continue
      pp = n*(x*p1-p2)/(x*x-1.D0)
      !
      Return
      End
C-----------------------------------------------------------------------
      !
C     ! Calculates the Legendre functions Q_n(x) and Q'_n(x).
C     ! Q_n(x) is in q1 and Q'_n(x) in qp.
      !
      Subroutine legq(q1,qp,x,n)
      Implicit Double Precision (A-z)
      Integer j, n
      !
      q2 = 0.5D0*dlog(dabs((1.D0+x)/(1.D0-x)))
      q1 = x*q2-1.D0
      Do 10 j=2,n
         q3 = q2
         q2 = q1
         q1 = ((2.D0*j-1.D0)*x*q2-(j-1.D0)*q3)/j
   10 Continue
      qp = n*(x*q1-q2)/(x*x-1.D0)
      !
      Return
      End
C-----------------------------------------------------------------------
