/* Investigate for which initial values, the zeroes 
   of the complex plane equation z^3 -1 converge to 
   the real solution z=1

   use standard Newton-Rapson                        */
/* gcc -O3 -o ex2.3.exe ex2.3.c -lm                  */
#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define NUM_RANIT  100000
#define MAXIT      200
#define SIDE       2.
#define EPS        1.e-12

double absval(double val);

int main(void)
{
  int i, count;
  int num_it;

  num_it = NUM_RANIT;
  for(i=0; i<num_it; i++)
    {
      int check=0;
      double real0, imm0;
      double real, imm;
      
      /* initial value, random assigned */
      real0 = 2.*SIDE*((double)(rand())/RAND_MAX) - 2.;
      imm0  = 2.*SIDE*((double)(rand())/RAND_MAX) - 2.;      
      real=real0;
      imm=imm0;
      
      /* implement Newton-Rapson */
      for(count=0; count<MAXIT; count++)
	{
	  double old_i, old_r;
	  double imm2, imm3, imm4, imm5;
	  double real2, real3, real4, real5;
	  double new_r, new_i, den;
	  double mod, new_mod;

	  imm2 = imm*imm;
	  imm3 = imm*imm2;
	  imm4 = imm2*imm2;
	  imm5 = imm2*imm3;
	  
	  real2 = real*real;
	  real3 = real*real2;
	  real4 = real2*real2;
	  real5 = real2*real3;
	  
	  den = 3.*(real2 + imm2)*(real2 + imm2);
	  new_r = 
	    ( 2.*real5 + 2.*real*imm4 + 4.*real3*imm2 + real2 - imm2 )/den;
	  new_i = 
	    ( 2.*imm5 + 2.*real4*imm + 4.*real2*imm3 - 2.*real*imm )/den;
	  
	  old_r = real;
	  old_i = imm;
	  mod = sqrt(real2+imm2);
	  real = new_r;
	  imm = new_i;
	  new_mod=sqrt(real*real+imm*imm);
	  /*if( (absval(new_mod-mod)<EPS))*/
	  if( (absval(real - old_r)<EPS) && (absval(imm - old_i)<EPS) )
	    {check=1; break;}
	}
      if(check==0)
	printf("# error,  maximum number of it.s exceeded\n");
      else if( ( absval(real -1.)<EPS) && (absval(imm)< EPS))
	printf("%f %f\n", real0, imm0 );
    }
  return 0;
}
/**************************************************************/
double absval(double val)
{
  double cmp;
  cmp = (val > 0) ? val: -val;
  return cmp;
}
/**************************************************************/
