C-----------------------------------------------------------------------
      !
C     ! Given a set of data points x(1:ndata), y(1:ndata) with individual
C     ! standard deviations sig(1:ndata), fit them to a straight line
C     ! y = a + b x by minimizing chi^2. Returned are a, b and their
C     ! respective probable uncertainties siga and sigb and the chi-square
C     ! chi2. If mwt = 0 on input, then the standard deviations are assumed
C     ! to be unavailable: the normalisation of chi^2 is to unit standard
C     ! deviation on all points.
      !
      Subroutine fit(x,y,ndata,sig,mwt,a,b,siga,sigb,chi2)
      Implicit Double Precision (A-z)
      Integer i, mwt, ndata
      Dimension sig(ndata), x(ndata), y(ndata)
      !
C     ! Initialise sums to zero.
      !
      sx = 0.D0
      sy = 0.D0
      st2 = 0.D0
      b = 0.D0
      !
C     ! Accumulate sums with weights,
      !
      If (mwt.ne.0) Then
         ss = 0.D0
         Do 11 i=1,ndata
            wt = 1.D0/(sig(i)**2)
            ss = ss+wt
            sx = sx+x(i)*wt
            sy = sy+y(i)*wt
   11    Continue
      !
C     ! or without weights
      !
      Else
         Do 12 i=1,ndata
            sx = sx+x(i)
            sy = sy+y(i)
   12    Continue
         ss = dble(ndata)
      End If
      !
      sxoss = sx/ss
      !
      If (mwt.ne.0) Then
         Do 13 i=1,ndata
            t = (x(i)-sxoss)/sig(i)
            st2 = st2+t*t
            b = b+t*y(i)/sig(i)
   13    Continue
      Else
         Do 14 i=1,ndata
            t = x(i)-sxoss
            st2 = st2+t*t
            b = b+t*y(i)
   14    Continue
      End If
      !
C     ! Solve for a, b, siga and sigb
      !
      b = b/st2
      a = (sy-sx*b)/ss
      siga = dsqrt((1.D0+sx*sx/(ss*st2))/ss)
      sigb = dsqrt(1.D0/st2)
      chi2 = 0.D0
      !
C     ! Calculate chi2
      !
      If (mwt.eq.0) Then
         Do 15 i=1,ndata
            chi2 = chi2+(y(i)-a-b*x(i))**2
   15    Continue
      !
C     ! For unweighted data evaluate typical sig using chi2, and
C     ! adjust the standard deviations
      !
         sigdat = sqrt(chi2/(ndata-2))
         siga = siga*sigdat
         sigb = sigb*sigdat
      Else
         Do 16 i=1,ndata
            chi2 = chi2+((y(i)-a-b*x(i))/sig(i))**2
   16    Continue
      End If
      Return
      End
C-----------------------------------------------------------------------
