C----------------------------------------------------------------------
      !
C     ! Given alf, the parameter of the Laguerre polynomial, this routine
C     ! returns arrays x(1:n) and w(1:n) of length n, containing the
C     ! abcissas and weights of the n-point Gauss-Laguerre quadrature
C     ! formula. The smallest abscissa is returned in x(1), the largest
C     ! in x(n).
      !
      Subroutine gaulag(x,w,n,alf)
      Implicit Double Precision (A-z) 
      Integer i, its, j, n, maxit
      Dimension x(n), w(n)
      Parameter (maxit = 30, eps = 3.D-14)
      !
C     USES gammln
      !
C     ! Loop over desired roots.
      !
      Do 30 i=1,n
         If (i.eq.1) Then
      !
C     ! Initial guess for smallest root.
      !
            z = (1.D0+alf)*(3.D0+0.92D0*alf)/(1.D0+2.4D0*n+1.8D0*alf)
         Else If (i.eq.2) Then
      !
C     ! Initial guess for second root.
      !
            z = z+(15.D0+6.25D0*alf)/(1.D0+0.9D0*alf+2.5D0*n)
         Else
      !
C     ! Initial guess for other roots.
      !
            ai = i-2
            z = z+((1.D0+2.55D0*ai)/(1.9D0*ai)+1.26D0*ai*alf/
     .          (1.D0+3.5D0*ai))*(z-x(i-2))/(1.D0+0.3D0*alf)
         End If
      !
C     ! Refinement by means of Newtons method.
      !
         Do 20 its=1,maxit
            p1 = 1.D0
            p2 = 0.D0
      !
C     ! Loop the recurrence relation to get the
C     ! Laguerre polynomial evaluated at z.
      !
            Do 10 j=1,n
               p3 = p2
               p2 = p1
               p1 = ((2*j-1+alf-z)*p2-(j-1+alf)*p3)/j
   10       Continue
      !
C     ! p1 is now the desired Laguerre polynomial. We next compute its
C     ! derivative pp, by a standard relation involving also p2, the
C     ! polynomial of one lower order.
      !
            pp = (n*p1-(n+alf)*p2)/z
            z1 = z
      !
C     ! Newtons method.
      !
            z = z1-p1/pp
            If (dabs(z-z1).le.eps) Goto 1
   20    Continue
         Pause 'too many iterations in gaulag'
      !
C     ! Store the roots and weights.
      !
    1    x(i) = z
         w(i) = -dexp(gammln(alf+n)-gammln(dble(n)))/(pp*n*p2)
   30 Continue
      !
      Return
      End
C-----------------------------------------------------------------------
