C----------------------------------------------------------------------
      !
C     ! Given the lower and upper limits of integration x1 and x2, and
C     ! given n, this routine returns arrays x(1:n) and w(1:n) of length
C     ! n, containing the abcissas and weights of the Gauss-Legendre
C     ! n-point quadrature formula.
      !
      Subroutine gauleg(x1,x2,x,w,n)
      Implicit Double Precision (A-z)
      Integer i, j, m, n
      Dimension x(n), w(n)
      Parameter (Pi = 3.141592653589793238D0, eps = 3.D-14)
      !
C     ! The roots are symmetric in the interval,
C     ! so we only have to find half of them.
      !
      m = int(n+1)/2
      xm = 0.5D0*(x2+x1)
      xl = 0.5D0*(x2-x1)
      !
C     ! Loop over the desired roots.
      !
      Do 30 i=1,m
      !
C     ! Starting point to find the i-th root
      !
         z = dcos(Pi*(i-0.25D0)/(n+0.5D0))
      !
   10    p1 = 1.D0
         p2 = 0.D0
      !
C     ! Loop up the recurrence relation to get the
C     ! Legendre polynomial evaluated at z.
      !
         Do 20 j=1,n
            p3 = p2
            p2 = p1
   20       p1 = ((2.D0*j-1.D0)*z*p2-(j-1.D0)*p3)/j
      !
C     ! Calculate pp the derivative of the Legendre polynomial, by
C     ! a standard relation involving p1 and p2, the polynomial of
C     ! one lower order.
      !
         pp = n*(z*p1-p2)/(z*z-1.D0)
         z1 = z
      !
C     ! Newtons method.
      !
         z = z1-p1/pp
         If (dabs(z-z1).gt.eps) Goto 10
      !
C     ! Scale the root to the desired interval
C     ! and put in symmetric counterpart.
      !
         x(i) = xm-xl*z
         x(n+1-i) = xm+xl*z
      !
C     ! Compute the weight and its symmetric counterpart.
      !
         w(i) = 2.D0*xl/((1.D0-z*z)*pp*pp)
         w(n+1-i) = w(i)
   30 Continue
      !
      Return
      End
C-----------------------------------------------------------------------
