      !
C     ! Program for calculating the energy levels
C     ! of a quantum mechanical square well potential
C     !
C     ! Employs the Newton-Raphson algorithm
C     ! 
C     ! Last change: 16.05.2003
C     ! Matthias Jamin
      !
      Program QM_Well_Potential
      Implicit Double Precision (A-z)
      External funcd
      !
C     ! Initial range for first root
      !
      x1 = 0.D0
      x2 = 1.D0
      !
C     ! Initial range for second root
      !
C     x1 = 4.D0
C     x2 = 8.D0
      !
      root = rtnewt(funcd,x1,x2,1.D-14)
      Write (*,*) root
      !
      Stop
      End
C----------------------------------------------------------------------
      !
C     ! Implementation of the Newton-Raphson algorithm
      !
      Double Precision Function rtnewt(funcd,x1,x2,xacc)
      Implicit Double Precision (A-z)
      Integer j, JMAX
      External funcd
      Parameter (JMAX=20)
      !
C     ! Initial guess for the root
      !
      rtnewt = 0.5D0*(x1+x2)
      !
C     ! Iteration loop up to JMAX iterations
      !
      Do 10 j=1,JMAX
         Call funcd(rtnewt,f,df)
         dx=f/df
         rtnewt = rtnewt-dx
         If ((x1-rtnewt)*(rtnewt-x2).lt.0.) Pause
     .'rtnewt jumped out of brackets'
         If (dabs(dx).lt.xacc) Return
   10 Continue
      !
      Pause 'rtnewt exceeded maximum iterations'
      !
      End
C----------------------------------------------------------------------
      !
C     ! Subroutine which provides the function to
C     ! be solved as well as its first derivative
      !
      Subroutine funcd(E,f,df)
      Implicit Double Precision (A-z)
      !
C     ! Initialise the constants
      !
      Data V0, a, hq2 /10.D0, 3.D0, 7.6199682D0/
      !
      al = dsqrt(2.D0*E/hq2)
      be = dsqrt(2.D0*(V0-E)/hq2)
      !
C     ! Function for which the root shall be calculated
      !
C     ! Function for even wave function
      f = al*dsin(al*a) - be*dcos(al*a)
      !
C     ! Function for odd wave function
C     f = al*dcos(al*a) + be*dsin(al*a)
      !
C     ! First derivative of the function
      !
      alp = al/2.D0/E
      bep = be/2.D0/(E-V0)
      !
C     ! Derivative for even wave function
      df = alp*(1.D0+be*a)*dsin(al*a) + (al*alp*a-bep)*dcos(al*a) 
      !
C     ! Derivative for odd wave function
C     df = alp*(1.D0+be*a)*dcos(al*a) - (al*alp*a-bep)*dsin(al*a) 
      !
      Return
      End
C----------------------------------------------------------------------

