C-----------------------------------------------------------------------
      !
C     ! Given arrays x(1:n) and y(1:n) containing a tabulated function
C     ! y_i = f(x_i), with x_1 < x_2 < ... < x_n, and given values yp1
C     ! and ypn for the first derivative of the interpolating function
C     ! at the points x_1 and x_n respectively, this routine returns an
C     ! array y2(1:n) of length n which contains the second derivatives
C     ! of the interpolating function at the tabulated points x_i. If
C     ! yp1 and/or ypn are equal to 10^30 or larger, the routine sets
C     ! the corresponding boundary condition to the natural spline with
C     ! zero second derivative on the boundary.
      !
      Subroutine spline(x,y,y2,n)
      Implicit Double Precision (A-z)
      Integer i, k, n
      Dimension x(n), y(n), y2(n), u(n)
      !
C     ! The lower boundary condition is either set to be natural,
      !
      If (yp1.gt.0.99D30) Then
         y2(1) = 0.D0
         u(1) = 0.D0
      !
C     ! or else to have the specified first derivative.
      !
      Else
         y2(1) = -0.5D0
         u(1) = (3.D0/(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      End If
      !
C     ! This is the decomposition loop of the tridiagonal algorithm.
C     ! y2 and u are used for temporary storage of the decomposed factors.
      !
      Do 11 i=2,n-1
         sig = (x(i)-x(i-1))/(x(i+1)-x(i-1))
         p = sig*y2(i-1)+2.D0
         y2(i) = (sig-1.D0)/p
         u(i) = (6.D0*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))
     .          /(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
   11 Continue
      !
C     ! The upper boundary condition is either set to be natural,
      !
      If (ypn.gt.0.99D30) Then
         qn = 0.D0
         un = 0.D0
      Else
      !
C     ! or else to have the specified first derivative.
      !
         qn = 0.5D0
         un = (3.D0/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      End If
      !
      y2(n) = (un-qn*u(n-1))/(qn*y2(n-1)+1.D0)
      !
C     ! This is the backsubstitution loop of the tridiagonal algorithm
      !
      Do 12 k=n-1,1,-1
         y2(k) = y2(k)*y2(k+1)+u(k)
12    Continue
      !
      Return
      End
C-----------------------------------------------------------------------
