C-----------------------------------------------------------------------
      !
C     ! Given the arrays xa(1:n) and ya(1:n) of length n, which tabulate
C     ! a function (with ordered xa_i's), and given the array y2a(1:n),
C     ! which is the output from spline above, and given a value of x,
C     ! this routine returns a cubic-spline interpolation value y.
      !
      Subroutine splint(xa,ya,y2a,x,y,n)
      Implicit Double Precision (A-z)
      Integer k, klo, khi, n
      Dimension xa(n), ya(n), y2a(n)
      !
      klo = 1
      khi = n
      !
C     ! We shall find the right place in the table by means of bisection.
C     ! This is optimal if sequential calls to this routine are at random
C     ! values of x. If sequential calls are in order, and closely spaced,
C     ! one would do better to store previous values of klo and khi and
C     ! test if the remain appropriate on the next call.
      !
1     If (khi-klo.gt.1) Then
         k = (khi+klo)/2
         If (xa(k).gt.x) Then
            khi = k
         Else
            klo = k
         End If
      Goto 1
      End If
      !
C     ! klo and khi now bracket the input value of x
      !
      h = xa(khi)-xa(klo)
      If (h.eq.0.D0) Pause 'Bad xa input.'
      a = (xa(khi)-x)/h
      b = (x-xa(klo))/h
      y = a*ya(klo)+b*ya(khi) +
     .    ((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.D0
      !
      Return
      End
C-----------------------------------------------------------------------
