(************** Content-type: application/mathematica **************

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     56492,       2135]*)
(*NotebookOutlinePosition[     87570,       3165]*)
(*  CellTagsIndexPosition[     87526,       3161]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Kompaktkurs"
}], "Title"],

Cell[CellGroupData[{

Cell["Inhalts\[UDoubleDot]bersicht", "Section"],

Cell[TextData[StyleBox["1. Einf\[UDoubleDot]hrung\n2. Arithmetik\n3. \
Fortgeschrittene Anwendungen\n4. Analysis\n5. L\[ODoubleDot]sen von \
Gleichungen\n6. Lineare Algebra\n7. Grafik\n8. Einige \
vollst\[ADoubleDot]ndige Beispiele",
  FontWeight->"Bold"]], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Title"],
  ". Einf\[UDoubleDot]hrung"
}], "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Allgemeines"
}], "Subsection"],

Cell[TextData[{
  "Moderne Computeralgebra-Systeme (CAS) wie ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", ",
  StyleBox["Maple",
    FontSlant->"Italic"],
  " oder ",
  StyleBox["MuPad ",
    FontSlant->"Italic"],
  "sind m\[ADoubleDot]chtige Mathe\[Dash]Programme, die neben dem numerischen \
Rechnen (mit beliebiger Genauigkeit) auch symbolische Mathematik und die \
Visualisierung von Ergebnissen beherrschen. Entgegen anderslautenden \
Bekundungen der Hersteller enthalten diese Pakete zwar nicht das \
mathematische Wissen der Welt und ersetzen auch keinen Mathematiker, aber sie \
k\[ODoubleDot]nnen einem immerhin viele zeitaufw\[ADoubleDot]ndige Arbeiten \
abnehmen."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Starten des Programmes und Erste Hilfe"
}], "Subsection"],

Cell[TextData[{
  "Unter Unix kann ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " von einem Terminal aus mit dem Kommando \"",
  Cell[BoxData[
      \(mathematica\)]],
  "\" gestartet werden. Das sollte normalerweise das graphische \
Benutzerinterface aktivieren. Mathematica ist ein interaktives System, das \
Eingaben vom Benutzer liest, diese auswertet, und das Ergebnis unmittelbar \
ausspuckt:"
}], "Text"],

Cell[BoxData[
    \(1 + 2\)], "Input",
  CellLabel->"In[1]:="],

Cell[TextData[{
  "Eingaben werden in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " immer durch gleichzeitiges Dr\[UDoubleDot]cken von \[ShiftKey] und \
\[ReturnIndicator] ausgewertet!"
}], "Note"],

Cell["\<\
Die Ausgabe des Ergebnisses kann man unterdr\[UDoubleDot]cken, \
indem man ein Semikolon an die Eingabe anh\[ADoubleDot]ngt. Vor allem bei \
umfangreichen Zwischenergebnissen hilft das, den \[CapitalUDoubleDot]berblick \
zu bewahren. \
\>", "Text"],

Cell[TextData[{
  "Dieses Handbuch ist zu kurz, um auch nur die vorgestellten Funktionen von \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " im Detail zu behandeln. F\[UDoubleDot]r umfangreichere Informationen zu \
bestimmten Themen ist daher die Online-Hilfe unentbehrlich. Informationen zu \
speziellen Funktionen lassen sich zus\[ADoubleDot]tzlich mit \"",
  StyleBox["?\"", "Input"],
  " abrufen. (Abh\[ADoubleDot]ngig von der verwendeten ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Version, erscheinen Teile des Hilfe-Outputs als anklickbare Verkn\
\[UDoubleDot]pfungen zur Online-Hilfe und zum ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Buch; wenn Sie Gl\[UDoubleDot]ck und die richtige Version haben wird die \
folgende Ausgabe mit einem  ",
  StyleBox["More... ",
    FontVariations->{"Underline"->True}],
  "  abgeschlossen. Falls Sie draufklicken,  werden Sie zu detaillierteren \
Informationen und Beispielen (!) weitergef\[UDoubleDot]hrt) :"
}], "Text"],

Cell[BoxData[
    \(\(?Log\)\)], "Input",
  CellLabel->"In[2]:="],

Cell["\<\
Um eine Liste aller Funktionen zu erhalten, deren Namen auf ein \
bestimmtes Muster passen, kann man ein Sternchen in den gesuchten Namen einf\
\[UDoubleDot]gen:\
\>", "Text"],

Cell[BoxData[
    \(\(?Eq*\)\)], "Input",
  CellLabel->"In[3]:="],

Cell[BoxData[
    \(\(?F*List\)\)], "Input",
  CellLabel->"In[4]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " als Taschenrechner"
}], "Subsection"],

Cell[TextData[{
  "Die Notation von Ausdr\[UDoubleDot]cken in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " orientiert sich an",
  " der gebr\[ADoubleDot]uchlichen mathematischen Notation. Eine g\
\[UDoubleDot]ltige Eingabe ist beispielsweise"
}], "Text"],

Cell[BoxData[
    \(\((2^12 + 7.1)\)*\((\(-1.5\)/2.5)\)\)], "Input",
  CellLabel->"In[5]:="],

Cell[TextData[{
  "Es gelten die gebr\[ADoubleDot]uchlichen Vorrangsregeln f\[UDoubleDot]r \
Operatoren, Ausdr\[UDoubleDot]cke k\[ODoubleDot]nnen aber auch mit ",
  StyleBox["runden",
    FontSlant->"Italic"],
  " Klammern explizit gruppiert werden",
  ". Funktionsargumente werden in ",
  StyleBox["eckige",
    FontSlant->"Italic"],
  " Klammern eingeschlossen, um sie von gew\[ODoubleDot]hnlichen Ausdr\
\[UDoubleDot]cken zu unterscheiden:"
}], "Text"],

Cell[BoxData[
    \(Sin[3.1]/Sqrt[2]\)], "Input",
  CellLabel->"In[6]:="],

Cell["\<\
Die Namen aller in Mathematica vordefinieren Funktionen, Prozeduren \
und Konstanten beginnen mit einem Gro\[SZ]buchstaben.\
\>", "Note"],

Cell[TextData[{
  "Das Multiplikations-Zeichen kann h\[ADoubleDot]ufig weggelassen werden, \
etwa ist ",
  Cell[BoxData[
      \(2  Sin[3  x]\)]],
  " gleichbedeutend mit ",
  Cell[BoxData[
      \(2*Sin[3*x]\)]],
  ". Auch Leerzeichen zwischen zwei (abgeschlossenen) Ausdr\[UDoubleDot]cken \
erwirken die Multikplikation der beiden: ",
  Cell[BoxData[
      \(x\ y\)]],
  " ist also ",
  Cell[BoxData[
      \(x*y\)]],
  ", ",
  Cell[BoxData[
      \(xy\)]],
  " (ohne Leerzeichen) wird dagegen als ",
  StyleBox["ein",
    FontSlant->"Italic"],
  " Symbol \"",
  Cell[BoxData[
      \(xy\)]],
  "\" gewertet."
}], "Text"],

Cell[TextData[{
  "Auf die letzen beiden Ausgaben kann mit ",
  Cell[BoxData[
      \(%\)]],
  " (letzte Ausgabe) und ",
  Cell[BoxData[
      \(%%\)]],
  " (vorletzte Ausgabe) Bezug genommen werden. Meist ist es aber sinnvoller, \
wichtige Zwischenergebnisse in Variablen zu speichern, um sp\[ADoubleDot]ter \
auf sie zur\[UDoubleDot]ckgreifen zu k\[ODoubleDot]nnen."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2. Arithmetik", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Exakte und Inexakte Arithmetik"
}], "Subsection"],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " unterscheidet zwischen ",
  StyleBox["exakten",
    FontSlant->"Italic"],
  " und ",
  StyleBox["inexakten",
    FontSlant->"Italic"],
  ", also gen\[ADoubleDot]herten numerischen Ausdr\[UDoubleDot]cken. Exakte \
Rechnungen werden mit beliebiger Genauigkeit durchgef\[UDoubleDot]hrt, bei \
inexakten kann es nach einigen Schritten zu Rundungsfehlern kommen. \
Kommazahlen und die ",
  StyleBox["N\[Dash]",
    FontFamily->"Courier"],
  "Funktion erzwingen die numerische Auswertung eines Ausdrucks. ",
  Cell[BoxData[
      \(N\)]],
  " nimmt als optionalen Parameter die Anzahl der signifikanten Stellen des \
Ergebnisses:"
}], "Text",
  AutoItalicWords->{
    "Mathematica", "MathLink", "MathSource", "MathReader", "MathScript"}],

Cell[BoxData[
    \(Factorial[50]\)], "Input",
  CellLabel->"In[7]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[8]:="],

Cell[BoxData[
    \(Sin[Pi/3]\)], "Input",
  CellLabel->"In[9]:="],

Cell[BoxData[
    \(N[%, 20]\)], "Input",
  CellLabel->"In[10]:="],

Cell[BoxData[
    \(Sin[Pi/3.0]\)], "Input",
  CellLabel->"In[11]:="],

Cell[TextData[{
  StyleBox["Vorsicht: ",
    FontSlant->"Italic"],
  "Die Regeln, die ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " f\[UDoubleDot]r numerische Rechnungen anwendet, sind kompliziert und k\
\[ODoubleDot]nnen zu \[UDoubleDot]berraschenden Ergebnissen \
f\[UDoubleDot]hren. Zudem ist umfangreiche Numerik ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " meist sehr langsam. In solchen F\[ADoubleDot]llen kann es von Vorteil \
sein, die eigentlichen Berechnungen in C\[Dash] oder Fortran\[Dash]Programme \
auszulagern."
}], "Text"],

Cell["\<\
Ein Beispiel, warum man der Numerik von Mathematica \
(\[UDoubleDot]brigens jeglicher Numerik) \.13nicht immer trauen sollte:\
\>", \
"Text"],

Cell[BoxData[{
    \(\(x = 1.11111111111111111111;\)\), "\[IndentingNewLine]", 
    \(\(Do[x = 2*x - x, {60}];\)\), "\[IndentingNewLine]", 
    \(x \[Equal] x + 1\)}], "Input",
  CellLabel->"In[12]:="],

Cell[BoxData[
    \(Clear[x]\)], "Input",
  CellLabel->"In[15]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Komplexe Zahlen"
}], "Subsection"],

Cell[TextData[{
  "Neben dem Rechnen mit ganzen, rationalen und reellen Zahlen unterst\
\[UDoubleDot]tzt Mathematica auch komplexe Arithmetik. Die \
imagin\[ADoubleDot]re Einheit wird als \"",
  Cell[BoxData[
      \(I\)]],
  "\" geschrieben. \n",
  StyleBox["Mathematica",
    FontVariations->{"CompatibilityType"->0}],
  " implementiert die f\[UDoubleDot]r komplexe Arithmetik wichtigen \
Funktionen Abs[z], Re[z], Im[z], Conjugate[z] und Arg[z]. Achtung: falls z in \
symbolischer Form als  z=x + y I  gegeben ist, wird angenommen, dass sowohl x \
als auch y selbst komplex sind !"
}], "Text"],

Cell[BoxData[
    \(z = 2\  + \ 3\ I\)], "Input",
  CellLabel->"In[16]:="],

Cell[BoxData[
    \(\(\(Abs[z]\)\(\ \ \)\)\)], "Input",
  CellLabel->"In[17]:="],

Cell[BoxData[
    \(\(\(Im[z]\)\(\ \)\)\)], "Input",
  CellLabel->"In[18]:="],

Cell[BoxData[
    \(Conjugate[z]\)], "Input",
  CellLabel->"In[19]:="],

Cell[BoxData[
    \(Arg[z]\)], "Input",
  CellLabel->"In[20]:="],

Cell[BoxData[
    \(z = x\  + \ y\ I\)], "Input",
  CellLabel->"In[21]:="],

Cell[BoxData[
    \(Re[z]\)], "Input",
  CellLabel->"In[22]:="],

Cell[BoxData[
    \(Simplify[Exp[x] \[Equal] Exp[x + 2  I\ Pi]]\)], "Input",
  CellLabel->"In[23]:="],

Cell[TextData[{
  "Wenn es n\[ODoubleDot]tig ist, zwischen exponentieller und \
trigonometrischer Schreibweise eines Ausdrucks zu wechseln, \
k\[ODoubleDot]nnen ",
  Cell[BoxData[
      \(ExpToTrig\)]],
  " und ",
  Cell[BoxData[
      \(TrigToExp\)]],
  " verwendet werden:"
}], "Text"],

Cell[BoxData[
    \(ExpToTrig[Exp[I\ x]]\)], "Input",
  CellLabel->"In[24]:="],

Cell[BoxData[
    \(TrigToExp[%^2]\)], "Input",
  CellLabel->"In[25]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "\.13",
  
  CounterBox["Subsection"],
  ". Mathematische Funktionen und Konstanten"
}], "Subsection"],

Cell["Alle gebr\[ADoubleDot]uchlichen mathematischen Konstanten...", "Text"],

Cell[BoxData[GridBox[{
        {\(Pi, \ \[AliasIndicator]pi\[AliasIndicator]\), 
          StyleBox["\[Pi]",
            FontFamily->"Times"]},
        {"E", 
          StyleBox[\(Eulersche\ Zahl\),
            FontFamily->"Times"]},
        {"I", 
          StyleBox[\(Imagin\[ADoubleDot]re\ Einheit\),
            FontFamily->"Times"]},
        {\(Infinity, \ \[AliasIndicator]inf\[AliasIndicator]\), 
          StyleBox[\(Symbol\ f\[UDoubleDot]r\ \[Infinity]\),
            FontFamily->"Times"]},
        {\(Degree, \ \[Degree]\), 
          StyleBox[\(Umrechnungsfaktor\ Grad \[Rule] Rad : \ 180/\[Pi]\),
            FontFamily->"Times"]},
        {\(Integers, \ Rationals, Reals\), 
          RowBox[{
            StyleBox["Zahlenmengen",
              FontFamily->"Times"], 
            StyleBox[" ",
              FontFamily->"Times"], 
            RowBox[{
              StyleBox["(",
                FontFamily->"Times"], 
              RowBox[{
                StyleBox["etwa",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["f\[UDoubleDot]r",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["Annahmen",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["in",
                  FontFamily->"Times"], 
                StyleBox[" ",
                  FontFamily->"Times"], 
                StyleBox["Simplify",
                  FontFamily->"Courier"]}], 
              StyleBox[")",
                FontFamily->"Times"]}]}]}
        }]], "2ColumnTable"],

Cell["... und Funktionen sind in Mathematica vordefiniert:", "Text"],

Cell[BoxData[GridBox[{
        {\(Sin[x], Cos[x], Tan[x]\), 
          StyleBox[\(Trigonometrische\ Funktionen\),
            FontFamily->"Times"]},
        {\(Sinh[x], Cosh[x], Tanh[x]\), 
          StyleBox[\(Hyperbolische\ Funktionen\),
            FontFamily->"Times"]},
        {\(Log[x], Log[b, x], Exp[x]\), 
          
          StyleBox[\(Logarithmus\ \((nat\[UDoubleDot]rlich/
                    zur\ Basis\ b)\), \ Exponentialfunktion\),
            FontFamily->"Times"]},
        {\(Sqrt[x]\), 
          StyleBox["Quadratwurzel",
            FontFamily->"Times"]},
        {\(Random[]\), 
          StyleBox[\(Zufallszahl\ zwischen\ 0\ und\ 1\),
            FontFamily->"Times"]}
        }]], "2ColumnTable"],

Cell[BoxData[
    \(\(?Random\)\)], "Input",
  CellLabel->"In[26]:="],

Cell[BoxData[
    \(\(?Arc*\)\)], "Input",
  CellLabel->"In[27]:="],

Cell[TextData[{
  "Neben diesen Standardfunktionen existieren viele spezielle Funktionen, wie \
die Gamma-Funktion ",
  Cell[BoxData[
      \(Gamma\)]],
  ", die Fehlerfunktion ",
  Cell[BoxData[
      \(Erf\)]],
  ", verschiedene Bessel\[Dash]Funktionen (",
  Cell[BoxData[
      \(BesselJ, BesselY, BesselI, BesselK\)]],
  ") oder die Kugelfl\[ADoubleDot]chfenfunktionen ",
  Cell[BoxData[
      \(SphericalHarmonicY\)]],
  ". Mehr Informationen gibt es in der Online\[Dash]Hilfe unter \"special \
functions\"."
}], "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["3. Fortgeschrittene Anwendungen", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Listen"
}], "Subsection"],

Cell[TextData[{
  "Mehrere Objekte k\[ODoubleDot]nnen mithilfe von ",
  StyleBox["Listen",
    FontSlant->"Italic"],
  " zusammengefasst werden. Die Grundrechenarten und viele andere Funktionen \
sind f\[UDoubleDot]r Listen ",
  StyleBox["elementweise",
    FontSlant->"Italic"],
  " definiert. Dadurch verhalten sich bspw. Listen von Zahlen wie Vektoren \
oder \"Listen von Listen\" wie Matrizen:"
}], "Text"],

Cell[BoxData[
    \(v = {1, 2, 3, 4}\)], "Input",
  CellLabel->"In[28]:="],

Cell[BoxData[
    \(w = \(-v\)*2\)], "Input",
  CellLabel->"In[29]:="],

Cell[BoxData[
    \(w + v\)], "Input",
  CellLabel->"In[30]:="],

Cell[TextData[{
  "Zum Erstellen neuer Listen ist die Funktion ",
  Cell[BoxData[
      \(Table\)]],
  " h\[ADoubleDot]ufig praktisch: Folgendes Beispiel liefert eine Liste mit f\
\[UDoubleDot]nf identischen Elementen"
}], "Text"],

Cell[BoxData[
    \(Table[Sin[2], {5}]\)], "Input",
  CellLabel->"In[31]:="],

Cell[TextData[{
  "Hier ist ein komplizierteres Beispiel. Das Ergebnis is eine 10\[Times]3\
\[Dash]Matrix, in der die ",
  Cell[BoxData[
      \(n\)]],
  "\[Dash]te Zeile die Zahlen ",
  Cell[BoxData[
      \(n, n\^2, n\^3\)]],
  " enth\[ADoubleDot]lt:"
}], "Text"],

Cell[BoxData[
    \(potenzen = Table[{n, n^2, n^3}, {n, 10}]\)], "Input",
  CellLabel->"In[32]:="],

Cell[TextData[{
  "Alle Befehle, die ganzzahlige Laufvariablen verwenden (vor allem ",
  Cell[BoxData[
      \(Table\)]],
  ", ",
  Cell[BoxData[
      \(Do\)]],
  ", ",
  Cell[BoxData[
      \(Sum\)]],
  ", ",
  Cell[BoxData[
      \(Product\)]],
  "), benutzen daf\[UDoubleDot]r dieselbe Syntax. M\[ODoubleDot]gliche \
Eingabeformen sind\n{nmax}\t\t\tnmax Wiederholungen ohne Laufvariable (erstes \
Beispiel)\n{n,nmax}\t\tn geht von 1 bis nmax in Schritten von 1\n\
{n,nmin,nmax}\tn geht von nmin bis nmax in Schritten von 1 (zweites Beispiel)\
\n{n,nmin,nmax,dn}\tn geht von nmin bis nmax in Schritten von dn"
}], "Note"],

Cell[TextData[{
  Cell[BoxData[
      \(Part[x, n]\)]],
  " liefert den n\[Dash]ten Eintrag einer Liste ",
  Cell[BoxData[
      \(x\)]],
  " zur\[UDoubleDot]ck. F\[UDoubleDot]r eine ",
  StyleBox["Matrix",
    FontSlant->"Italic"],
  " ",
  Cell[BoxData[
      \(x\)]],
  " ist das eine Liste, n\[ADoubleDot]mlich die n\[Dash]te ",
  StyleBox["Zeile.",
    FontSlant->"Italic"],
  " Um die n\[Dash]te ",
  StyleBox["Spalte",
    FontSlant->"Italic"],
  " zu erhalten, muss ",
  Cell[BoxData[
      \(x\)]],
  " vorher transponiert werden:"
}], "Text"],

Cell[BoxData[
    \(qzahlen = Part[Transpose[potenzen], 2]\)], "Input",
  CellLabel->"In[33]:="],

Cell[TextData[{
  "Statt ",
  Cell[BoxData[
      \(Part\)]],
  " k\[ODoubleDot]nnen auch doppelte eckige Klammern zum Indizieren von \
Listen verwendet werden. In beiden F\[ADoubleDot]llen hat das erste Element \
den Index 1 (nicht etwa 0 wie in \"C\")."
}], "Text"],

Cell[BoxData[
    \({potenzen[\([3]\)], potenzen[\([4, 3]\)]}\)], "Input",
  CellLabel->"In[34]:="],

Cell[TextData[{
  "Die Funktion ",
  Cell[BoxData[
      \(Apply\)]],
  " wendet eine Funktion auf alle Listenlemente ",
  StyleBox["auf einmal",
    FontSlant->"Italic"],
  " an"
}], "Text"],

Cell[BoxData[
    \(\(\(Apply[Plus, qzahlen]\)\(\ \ \ \ \ \ \)\( (*\ 
      Summiere\ alle\ Elemente\ *) \)\)\)], "Input",
  CellLabel->"In[35]:="],

Cell[TextData[{
  Cell[BoxData[
      \(Map\)]],
  " dagegen wendet eine Funktion auf jedes Listenelement ",
  StyleBox["einzeln",
    FontSlant->"Italic"],
  " an"
}], "Text"],

Cell[BoxData[
    \(Map[Sin, qzahlen]\)], "Input",
  CellLabel->"In[36]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Selbstdefinierte Variablen und Funktionen"
}], "Subsection"],

Cell["\<\
Symbole k\[ODoubleDot]nnen verwendet werden, um Funktionen zu \
definieren oder beliebige Ausdr\[UDoubleDot]cke zu speichern. In allen sp\
\[ADoubleDot]teren Rechnungen wird das Symbol automatisch durch den \
zugewiesenen Wert ersetzt:\
\>", "Text"],

Cell[BoxData[
    \(\(\(rand\)\(=\)\(Random[]\)\(\t\)\( (*\ 
        setze\ rand\ auf\ eine\ feste\ Zufallszahl\ *) \)\)\)], "Input",
  CellLabel->"In[37]:="],

Cell[BoxData[
    \(rand*2\)], "Input",
  CellLabel->"In[38]:="],

Cell[TextData[{
  "Eigene ",
  StyleBox["Funktionen",
    FontSlant->"Italic"],
  " k\[ODoubleDot]nnen auf \[ADoubleDot]hnliche Weise definiert werden:"
}], "Text"],

Cell[BoxData[
    \(randint[min_, max_] := Random[Integer, {min, max}]\)], "Input",
  CellLabel->"In[39]:="],

Cell[BoxData[
    \(Table[randint[2, 10], {10}]\)], "Input",
  CellLabel->"In[40]:="],

Cell[TextData[{
  "Zwei Punkte sind in diesem Beispiel wichtig: Der Unterstrich \"",
  Cell[BoxData[
      \(_\)]],
  "\" hinter ",
  StyleBox["min",
    FontFamily->"Courier"],
  " und ",
  StyleBox["max",
    FontFamily->"Courier"],
  " und das \"",
  Cell[BoxData[
      \( := \)]],
  "\" Symbol anstelle von \"",
  Cell[BoxData[
      \( = \)]],
  "\" bei der Variablendefinition."
}], "Text"],

Cell[TextData[{
  "Variablen werden mit einem einfachen \"",
  Cell[BoxData[
      \( = \)]],
  "\" definiert, Funktionen dagegen mit \"",
  Cell[BoxData[
      \( := \)]],
  "\". Auf der ",
  StyleBox["linken",
    FontSlant->"Italic"],
  " Seite der Zuweisung muss an die Argumentnamen einer Funktion ein \
Unterstrich angeh\[ADoubleDot]ngt werden."
}], "Note"],

Cell["Ungew\[UDoubleDot]nschte Definitionen k\[ODoubleDot]nnen wieder gel\
\[ODoubleDot]scht werden", "Text"],

Cell[BoxData[
    \(\(\(Clear[v]\)\(\t\t\)\( (*\ 
      ein\ Symbol\ l\[ODoubleDot]schen\ *) \)\)\)], "Input",
  CellLabel->"In[41]:="],

Cell[BoxData[
    \(\(\(Clear[r1, r2]\)\(\t\)\( (*\ 
      mehrere\ Symbole\ l\[ODoubleDot]schen\ *) \)\)\)], "Input",
  CellLabel->"In[42]:="],

Cell[BoxData[
    \(\(\(Clear["\<Global`*\>"]\)\(\ \)\( (*\ 
      alle\ selbstdefinierten\ Symbole\ l\[ODoubleDot]schen\ *) \)\)\)], \
"Input",
  CellLabel->"In[43]:="],

Cell[TextData[{
  "Eine spezielle Notation f\[UDoubleDot]r Funktionen ist im Umgang mit \
Operationen wie ",
  Cell[BoxData[
      \(Map\)]],
  " oder ",
  Cell[BoxData[
      \(Nest\)]],
  " oft sehr praktisch. ",
  StyleBox["Pure functions",
    FontSlant->"Italic"],
  " sind Funktionen ohne Namen, die \[UDoubleDot]berall dort direkt eingef\
\[UDoubleDot]gt werden k\[ODoubleDot]nnen, wo sonst ein Funktionsname \
erwartet w\[UDoubleDot]rde. Statt"
}], "Text"],

Cell[BoxData[
    \(f[x_] = Sin[x]^2\)], "Input",
  CellLabel->"In[44]:="],

Cell[BoxData[
    \(N[Map[f, Range[1, 10]]]\)], "Input",
  CellLabel->"In[45]:="],

Cell["kann man mit pure functions auch k\[UDoubleDot]rzer schreiben", "Text"],

Cell[BoxData[
    \(N[Map[Sin[#]^2 &, Range[1, 10]]]\)], "Input",
  CellLabel->"In[46]:="],

Cell[TextData[{
  "Hierbei werden die Funktionsargumente mit ",
  Cell[BoxData[
      \(#\)]],
  ", ",
  Cell[BoxData[
      \(#2\)]],
  ", ",
  Cell[BoxData[
      \(#3\)]],
  ", \[Ellipsis] bezeichnet und die Definition der Funktion mit einem \
Ampersand \"",
  Cell[BoxData[
      \( &\)]],
  "\" beendet."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Symbolische Mathematik, Umformen von Ausdr\[UDoubleDot]cken"
}], "Subsection"],

Cell[TextData[{
  "Symbole, denen noch kein Wert zugewiesen wurde, verhalten sich wie ",
  StyleBox["Unbekannte",
    FontSlant->"Italic"],
  ". Sie k\[ODoubleDot]nnen verwendet werden, um allgemeine Berechnungen \
vorzunehmen"
}], "Text"],

Cell[BoxData[
    \(Sum[q^n, {n, 1, Infinity}]\)], "Input",
  CellLabel->"In[47]:="],

Cell[BoxData[
    \(Integrate[Tan[c*y], y]\)], "Input",
  CellLabel->"In[48]:="],

Cell[TextData[{
  "Einige offensichtliche Vereinfachungen werden automatisch vorgenommen, in \
komplizierteren F\[ADoubleDot]llen muss man ",
  Cell[BoxData[
      \(Simplify\)]],
  " verwenden"
}], "Text"],

Cell[BoxData[
    \(Simplify[Sin[x]^2 + Cos[x]^2]\)], "Input",
  CellLabel->"In[49]:="],

Cell[BoxData[
    \(Simplify[\((a^2 - b^2)\)/\((a - b)\)]\)], "Input",
  CellLabel->"In[50]:="],

Cell[TextData[{
  "Manche symbolischen Ausdr\[UDoubleDot]cke k\[ODoubleDot]nnen nur unter \
bestimmten Annahmen vereinfacht werden. Zum Beispiel gilt ",
  Cell[BoxData[
      \(Sqrt[x^2] \[Equal] x\)]],
  " nur f\[UDoubleDot]r reelle ",
  Cell[BoxData[
      \(x\)]],
  ". In solchen F\[ADoubleDot]llen m\[UDoubleDot]ssen die Annahmen explizit \
an ",
  Cell[BoxData[
      \(Simplify\)]],
  " \[UDoubleDot]bergeben werden:"
}], "Text"],

Cell[BoxData[
    \(Simplify[Sqrt[x^2]]\)], "Input",
  CellLabel->"In[51]:="],

Cell[BoxData[
    \(Simplify[Sqrt[x^2], x < 0]\)], "Input",
  CellLabel->"In[52]:="],

Cell[BoxData[
    \(Simplify[Sin[x + 2  Pi\ n], n \[Element] Integers]\)], "Input",
  CellLabel->"In[53]:="],

Cell[TextData[{
  "Manchmal findet ",
  Cell[BoxData[
      \(FullSimplify\)]],
  " zus\[ADoubleDot]tzliche Vereinfachungen, die ",
  Cell[BoxData[
      \(Simplify\)]],
  " nicht entdeckt"
}], "Text"],

Cell[BoxData[
    \(Simplify[\((I/4)\)/E^\((2*I*x)\) - I/4*E^\((2*I*x)\)]\)], "Input",
  CellLabel->"In[54]:="],

Cell[BoxData[
    \(FullSimplify[%]\)], "Input",
  CellLabel->"In[55]:="],

Cell[TextData[{
  Cell[BoxData[
      \(Sqrt[x^2]\)]],
  " konnte in obigem Beispiel nicht vereinfacht werden, da ",
  Cell[BoxData[
      \(x\)]],
  " als komplexe Variable angenommen wurde, f\[UDoubleDot]r die die gew\
\[ODoubleDot]hnlichen Potenzregeln nur eingeschr\[ADoubleDot]nkt gelten. Die \
Anwendung dieser Potenzregeln kann mit ",
  Cell[BoxData[
      \(PowerExpand\)]],
  " erzwungen werden"
}], "Text"],

Cell[BoxData[
    \(PowerExpand[{\@x^2, \((a*b)\)^c, \((a^b)\)^c}]\)], "Input",
  CellLabel->"In[56]:="],

Cell[TextData[{
  "Symbole in Ausdr\[UDoubleDot]cken k\[ODoubleDot]nnen mithilfe des ",
  Cell[BoxData[
      \( /. \)]],
  " Operators ersetzt werden (",
  StyleBox["unentbehrlich!",
    FontWeight->"Bold"],
  ")"
}], "Text"],

Cell[BoxData[
    \(\@a /. a \[Rule] x^2 + y^2\)], "Input",
  CellLabel->"In[57]:="],

Cell[BoxData[
    \(a + b /. {a \[Rule] b^2, b \[Rule] c^3}\)], "Input",
  CellLabel->"In[58]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Schleifen und If-Anweisungen"
}], "Subsection"],

Cell[TextData[{
  "Die wichtigsten aus imperativen Programmiersprachen bekannten \
Schleifentypen sind auch in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " vorhanden. (Die von ",
  Cell[BoxData[
      \(Do\)]],
  " verwendeten Z\[ADoubleDot]hlvariablen wurden im Abschnitt \"Listen\" erkl\
\[ADoubleDot]rt.)"
}], "Text"],

Cell[BoxData[
    \(Do[Print["\<Hallo Welt!\>"], {5}]\)], "Input",
  CellLabel->"In[59]:="],

Cell[BoxData[{
    \(\(liste = {};\)\), "\[IndentingNewLine]", 
    \(Do[liste = Append[liste, \ Prime[n]], {n, 1000, 
        1005}]\), "\[IndentingNewLine]", 
    \(liste\)}], "Input",
  CellLabel->"In[60]:="],

Cell[TextData[{
  "Schleifen k\[ODoubleDot]nnen auch mit ",
  Cell[BoxData[
      \(For\)]],
  " und ",
  Cell[BoxData[
      \(While\)]],
  " formuliert werden."
}], "Text"],

Cell[TextData[{
  "H\[ADoubleDot]ufig ist es einfacher und schneller anstelle dieser \
allgemeinen Schleifen spezialisierte Operationen wie ",
  Cell[BoxData[
      \(Table\)]],
  ", ",
  Cell[BoxData[
      \(Map\)]],
  ", ",
  Cell[BoxData[
      RowBox[{"Nest", Cell[""]}]]],
  ", oder ",
  Cell[BoxData[
      \(Apply\)]],
  " zu verwenden"
}], "Text"],

Cell[BoxData[
    \(Table[Prime[i], {i, 1000, 1005}]\)], "Input",
  CellLabel->"In[63]:="],

Cell[BoxData[
    \(Map[Prime, Range[1000, 1005]]\)], "Input",
  CellLabel->"In[64]:="],

Cell[BoxData[
    \(Nest[Sin, x, 3]\)], "Input",
  CellLabel->"In[65]:="],

Cell[BoxData[
    \(Nest[Sin, 1.2, 3]\)], "Input",
  CellLabel->"In[66]:="],

Cell[BoxData[
    \(NestList[Sin, x, 3]\)], "Input",
  CellLabel->"In[67]:="],

Cell[TextData[{
  Cell[BoxData[
      \(If\)]],
  "-Anweisung funktionieren ganz \[ADoubleDot]hnlich wie in anderen \
Programmiersprachen auch"
}], "Text"],

Cell[BoxData[
    \(wuerfel[] := Random[Integer, {1, 6}]\)], "Input",
  CellLabel->"In[68]:="],

Cell[BoxData[
    \(If[wuerfel[] \[Equal] 
        wuerfel[], \[IndentingNewLine]Print["\<Pasch\>"], \
\[IndentingNewLine]Print["\<Kein Pasch\>"]]\)], "Input",
  CellLabel->"In[69]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Eingabe von Formeln"
}], "Subsection"],

Cell[TextData[{
  "Die Verwendung zweidimensionaler Formeln (etwa ",
  Cell[BoxData[
      \(\@2\)]],
  " anstelle von ",
  Cell[BoxData[
      \(Sqrt[2]\)]],
  ") und mathematischer Symbole kann die Notation manchmal merklich \
vereinfachen. F\[UDoubleDot]r die gebr\[ADoubleDot]uchlichsten Symbole stehen \
im File\[Dash]Men\[UDoubleDot] einige ",
  StyleBox["Paletten",
    FontSlant->"Italic"],
  " zur Verf\[UDoubleDot]gung (in den meisten F\[ADoubleDot]llen reicht \
\"BasicInput\"). Zwischen den Platzhaltern \[SelectionPlaceholder] und \
\[Placeholder] kann mit der \[TabKey]-Taste hin- und hergewechselt werden."
}], "Text",
  CellMargins->{{Inherited, Inherited}, {0, Inherited}},
  CellBracketOptions->{"Color"->RGBColor[0, 0.2, 1]},
  InputAutoReplacements->{"TeX"->StyleBox[ 
          RowBox[ {"T", 
            AdjustmentBox[ 
            "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
              0.5], "X"}]], "LaTeX"->StyleBox[ 
          RowBox[ {"L", 
            StyleBox[ 
              AdjustmentBox[ 
              "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
            AdjustmentBox[ 
            "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
              0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
      "MMA"->"Mathematica"},
  ParagraphSpacing->{0, 8},
  ScriptMinSize->9,
  CounterIncrements->"Text"],

Cell[TextData[{
  "Folgende Tabelle enth\[ADoubleDot]lt einige n\[UDoubleDot]tzliche \
Tastenkombinationen. Das Symbol \"\[AliasIndicator]\" steht dabei f\
\[UDoubleDot]r die \[EscapeKey]-Taste, das \"+\" deutet an, dass die Tasten \
",
  StyleBox["gleichzeitig",
    FontSlant->"Italic"],
  " zu dr\[UDoubleDot]cken sind."
}], "Text"],

Cell[BoxData[GridBox[{
        {"\[TabKey]", 
          
          StyleBox[\(zum\ n\[ADoubleDot]chsten\ Platzhalten\ \((\
\[SelectionPlaceholder]\ oder\ \[Placeholder])\)\ springen\),
            FontFamily->"Times"]},
        {\(\[ControlKey] + 2\), 
          RowBox[{
            StyleBox["Wurzel",
              FontFamily->"Times"], 
            StyleBox["-",
              FontFamily->"Times"], 
            RowBox[{
              StyleBox["Zeichen",
                FontFamily->"Times"], 
              StyleBox[" ",
                FontFamily->"Times"], 
              StyleBox["einf\[UDoubleDot]gen",
                FontFamily->"Times"], 
              StyleBox[" ",
                FontFamily->"Times"], \(\@a\)}]}]},
        {\(\[ControlKey] + \[ShiftKey] + 6\), 
          RowBox[{
            StyleBox["Bruch",
              FontFamily->"Times"], 
            StyleBox[" ",
              FontFamily->"Times"], 
            StyleBox["einf\[UDoubleDot]gen",
              FontFamily->"Times"], 
            StyleBox[" ",
              FontFamily->"Times"], 
            FractionBox[
              StyleBox["a",
                FontFamily->"Times"], "b"]}]},
        {\(\[ControlKey] + ^\), 
          RowBox[{
            StyleBox["Exponenten",
              FontFamily->"Times"], 
            StyleBox[" ",
              FontFamily->"Times"], 
            StyleBox["einf\[UDoubleDot]gen",
              FontFamily->"Times"], 
            StyleBox[" ",
              FontFamily->"Times"], 
            SuperscriptBox[
              StyleBox["a",
                FontFamily->"Times"], "b"]}]},
        {\(\[ControlKey] + \[SpaceKey]\), \(zur\ n\[ADoubleDot]chsth\
\[ODoubleDot]heren\ Schachtelungsebene\ gehen\)},
        {\(\[AliasIndicator]a\[AliasIndicator],  ... , \ \[AliasIndicator]z \
\[AliasDelimiter]\), 
          StyleBox[\(\[Alpha],  ... , \ \[Zeta]\),
            FontFamily->"Times"]},
        {\(\[AliasIndicator]inf \[AliasDelimiter]\), 
          StyleBox["\[Infinity]",
            FontFamily->"Times"]},
        {"\[AliasIndicator]elem\[AliasIndicator]", "\[Element]"},
        {"->", "\[Rule]"}
        }]], "2ColumnTable",
  FontFamily->"Times"],

Cell["Einen Ausdruck wie", "Text"],

Cell[BoxData[
    \(\(1\/\(\[Pi] \@\( x \((1 - x)\)\) + y\);\)\)], "Input",
  CellLabel->"In[70]:="],

Cell["\<\
erh\[ADoubleDot]lt man also entweder mithilfe der Paletten und \
\[TabKey], oder durch die Tastenfolge 1, \[ControlKey]+\[ShiftKey]+6, \
\[EscapeKey], pi, \[EscapeKey], \[ControlKey]+2, x(1-x), \[ControlKey]+\
\[SpaceKey], +y.\
\>", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4. Analysis", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Differentiation und Integration"
}], "Subsection"],

Cell[TextData[{
  "Ableiten nach einer Unbekannten ",
  Cell[BoxData[
      \(x\)]]
}], "Text"],

Cell[BoxData[
    \(D[x\ Log[x], x]\)], "Input",
  CellLabel->"In[71]:="],

Cell[TextData[{
  "Zweifache Ableitung nach ",
  Cell[BoxData[
      \(x\)]]
}], "Text"],

Cell[BoxData[
    \(D[Exp[x^2], {x, 2}]\)], "Input",
  CellLabel->"In[72]:="],

Cell[TextData[{
  "Gemischte Ableitung ",
  Cell[BoxData[
      \(\[PartialD]\_y\[PartialD]\_x\)]]
}], "Text"],

Cell[BoxData[
    \(D[Sin[x\ y], x, y]\)], "Input",
  CellLabel->"In[73]:="],

Cell["Unbestimmtes Integral...", "Text"],

Cell[BoxData[
    \(Integrate[1\/\(1 + x^2\), x]\)], "Input",
  CellLabel->"In[74]:="],

Cell[TextData[{
  "und bestimmtes Integral werden mit ",
  Cell[BoxData[
      \(Integrate\)]],
  " berechnet. Im folgenden Fall ist das Ergebnis allerdings nicht \
geschlossen darstellbar:"
}], "Text"],

Cell[BoxData[
    \(Integrate[Sin[Sin[x]], {x, 0, 10}]\)], "Input",
  CellLabel->"In[75]:="],

Cell["Numerische Integration l\[ODoubleDot]st das Problem", "Text"],

Cell[BoxData[
    \(NIntegrate[Sin[Sin[x]], {x, 0, 10}]\)], "Input",
  CellLabel->"In[76]:="],

Cell["\<\
Mehrfache Integrale lasssen sich einfach durch die Angabe mehrere \
Integrationsvariablen berechnen\
\>", "Text"],

Cell[BoxData[
    \(Integrate[Exp[\(-\((x^2 + y^2)\)\)], {x, 0, x1}, {y, 0, x2}]\)], "Input",\

  CellLabel->"In[77]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Reihenentwicklungen"
}], "Subsection"],

Cell[TextData[{
  "Analytische Funktionen k\[ODoubleDot]nnen mit ",
  Cell[BoxData[
      \(Series\)]],
  " in eine Potenzreihe entwickelt werden. Neben dem Entwicklungspunkt muss \
au\[SZ]erdem noch angegeben werden, bis zu welcher Ordnung entwickelt werden \
soll (hier 6 bzw. 4)"
}], "Text"],

Cell[BoxData[
    \(Series[Sin[x], {x, 0, 6}]\)], "Input",
  CellLabel->"In[78]:="],

Cell[BoxData[
    \(Series[Log[x + 1], {x, 0, 4}]\)], "Input",
  CellLabel->"In[79]:="],

Cell["\<\
Mit Potenzreihen kann in gewissen Grenzen wie mit anderen Ausdr\
\[UDoubleDot]cken gerechnet werden\
\>", "Text"],

Cell[BoxData[
    \(% + %%\)], "Input",
  CellLabel->"In[80]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Summen, Produkte und Grenzwerte"
}], "Subsection"],

Cell["\<\
Zus\[ADoubleDot]tzlich k\[ODoubleDot]nnen auch Grenzwerte, \
(unendliche) Reihen und (unendliche) Produkte ausgewertet werden\
\>", "Text"],

Cell[BoxData[
    \(Limit[Sin[x]/x, x \[Rule] 0]\)], "Input",
  CellLabel->"In[81]:="],

Cell[BoxData[
    \(Sum[Binomial[n, m], {m, 0, n}]\)], "Input",
  CellLabel->"In[82]:="],

Cell[BoxData[
    \(Sum[1/n^2, {n, 1, \[Infinity]}]\)], "Input",
  CellLabel->"In[83]:="],

Cell[BoxData[
    \(Product[1 + 1/k, {k, 1, n}]\)], "Input",
  CellLabel->"In[84]:="],

Cell[BoxData[
    \(Product[1 + 1/k^2, {k, 1, \[Infinity]}]\)], "Input",
  CellLabel->"In[85]:="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["5. L\[ODoubleDot]sen von Gleichungen", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Gew\[ODoubleDot]hnliche Gleichungen"
}], "Subsection"],

Cell[TextData[{
  "Um die L\[ODoubleDot]sung einer beliebigen Gleichung zu finden, steht mit \
",
  Cell[BoxData[
      \(FindRoot\)]],
  " ein numerischer Gleichungsl\[ODoubleDot]ser zur Verf\[UDoubleDot]gung"
}], "Text"],

Cell[BoxData[
    \(FindRoot[Sin[x] \[Equal] x^2, {x, 1}]\)], "Input",
  CellLabel->"In[86]:="],

Cell[TextData[{
  "Man kann den Suchbereich von ",
  Cell[BoxData[
      \(FindRoot\)]],
  " auf ein beliebiges Intervall einschr\[ADoubleDot]nken, indem man zus\
\[ADoubleDot]tzlich zum Startwert die Intervallgrenzen \[UDoubleDot]bergibt \
(obwohl das meistens nicht n\[ODoubleDot]tig ist):"
}], "Text"],

Cell[BoxData[
    \(FindRoot[Exp[x] \[Equal] 5  x^3, {x, 1, 0, 2}]\)], "Input",
  CellLabel->"In[87]:="],

Cell[TextData[{
  "Im Gegensatz zu ",
  Cell[BoxData[
      \(FindRoot\)]],
  " versucht ",
  Cell[BoxData[
      \(Solve\)]],
  ", die Gleichung aufzul\[ODoubleDot]sen und eine allgemeine \
L\[ODoubleDot]sung zu bestimmen. Wo das nicht geht (etwa im ersten Beispiel \
oben) gibt es eine Fehlermeldung, sonst eine Liste der L\[ODoubleDot]sungen"
}], "Text"],

Cell[BoxData[
    \(Solve[x^2 + p\ x + q \[Equal] 0, x]\)], "Input",
  CellLabel->"In[88]:="],

Cell[TextData[{
  "Auch wenn ",
  Cell[BoxData[
      \(Solve\)]],
  " alle L\[ODoubleDot]sungen findet, sind diese oft nur numerisch \
darstellbar"
}], "Text"],

Cell[BoxData[
    \(Solve[x^6 + x == 10, x]\)], "Input",
  CellLabel->"In[89]:="],

Cell[BoxData[
    \(N[%]\)], "Input",
  CellLabel->"In[90]:="],

Cell[TextData[{
  "Stattdessen kann man auch gleich die numerische Variante von ",
  Cell[BoxData[
      \(Solve\)]],
  " benutzen"
}], "Text"],

Cell[BoxData[
    \(NSolve[x^6 + x \[Equal] 10, x]\)], "Input",
  CellLabel->"In[91]:="],

Cell["\<\
Die vorgestellten Funktionen sind auch in der Lage, \
Gleichungssysteme in mehreren Variablesn zu l\[ODoubleDot]sen. Ein Beispiel\
\
\>", "Text"],

Cell[BoxData[
    \(FindRoot[{Sin[x] \[Equal] Cos[x], 
        x^2 + y^2 \[Equal] 1}, {x,  .5}, {y,  .2}]\)], "Input",
  CellLabel->"In[92]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Differentialgleichungen"
}], "Subsection"],

Cell["\<\
Schlie\[SZ]lich k\[ODoubleDot]nnen auch Differentialgleichungen und \
Systemen von Differentialgleichungen sowohl exakt als auch numerisch gel\
\[ODoubleDot]st werden.\
\>", "Text"],

Cell[BoxData[
    \(DSolve[\(y''\)[x] == \[Omega]^2\ y[x], y[x\ ], x]\)], "Input",
  CellLabel->"In[93]:="],

Cell[BoxData[
    \(DSolve[{\(y'\)[0] \[Equal] a, \(y''\)[x] \[Equal] \[Omega]^2  y[x]}, 
      y[x], x]\)], "Input",
  CellLabel->"In[94]:="],

Cell[TextData[{
  "In dieser Form wird eine Ersetzungsregel f\[UDoubleDot]r ",
  Cell[BoxData[
      \(y[x]\)]],
  " zur\[UDoubleDot]ckgeliefert. Ist man auch an Ableitungen von ",
  Cell[BoxData[
      \(y\)]],
  " interessiert, ist vorteilhaft nicht nach ",
  Cell[BoxData[
      \(y[x]\)]],
  ", sondern nach ",
  Cell[BoxData[
      \(y\)]],
  " aufzul\[ODoubleDot]sen:"
}], "Text"],

Cell[BoxData[
    \({y[x], \(y'\)[x]} /. 
      DSolve[{y[0] \[Equal] 0, \(y'\)[0] \[Equal] 
            a, \(y''\)[x] \[Equal] \[Omega]^2  y[x]}, y, x]\)], "Input",
  CellLabel->"In[95]:="],

Cell[TextData[{
  "Auch f\[UDoubleDot]r Differentialgleichungen k\[ODoubleDot]nnen numerische \
L\[ODoubleDot]sungen gesucht werden. ",
  Cell[BoxData[
      \(NDSolve\)]],
  " berechnet einige L\[ODoubleDot]sungspunkte im angegebenen Intervall, die \
von der zur\[UDoubleDot]ckgelieferten ",
  Cell[BoxData[
      \(InterpolatingFunction\)]],
  " interpoliert werden."
}], "Text"],

Cell[BoxData[
    \(NDSolve[{\(y'\)[x] \[Equal] Cos[x*y[x]], y[0] \[Equal] 1}, 
      y[x], {x, \(-5\), 5}]\)], "Input",
  CellLabel->"In[96]:="],

Cell["\<\
Mit folgender Ersetzung kann man dieses Ergebnis wie eine regul\
\[ADoubleDot]re Funktion benutzen (etwa zum Plotten).\
\>", "Text"],

Cell[BoxData[
    \(f[x_] := Evaluate[y[x] /. %]\)], "Input",
  CellLabel->"In[97]:="],

Cell["\<\
In \[ADoubleDot]hnlicher Weise lassen sich auch partielle \
Differentialgleichungen l\[ODoubleDot]sen. Mehr Informationen dazu gibt es \
(wieder einmal) in der Online-Hilfe.\
\>", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["6. Lineare Algebra", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Matrizen und Vektoren"
}], "Subsection"],

Cell["\<\
Wie bereits erl\[ADoubleDot]utert, verhalten sich Listen weitgehend \
wie Vektoren, da alle Grundrechenarten f\[UDoubleDot]r sie elementweise \
definiert sind. Matrizen werden als Listen von Listen dargestellt\
\>", "Text"],

Cell[BoxData[
    \(a = {a1, a2, a3}; \ b = {b1, b2, b3};\)], "Input",
  CellLabel->"In[98]:="],

Cell[BoxData[
    \(\(\(a . b\)\(\t\)\( (*\ Skalarprodukt\ *) \)\)\)], "Input",
  CellLabel->"In[99]:="],

Cell[BoxData[
    \(\(\(Cross[a, b]\)\(\t\)\( (*\ Kreuzprodukt\ *) \)\)\)], "Input",
  CellLabel->"In[100]:="],

Cell[BoxData[
    \(m = {{2, 1, 0}, {0, 3, \(-1\)}, {0, 0, 3}}\)], "Input",
  CellLabel->"In[101]:="],

Cell[BoxData[
    \(\(\(%\)\(//\)\(MatrixForm\)\(\t\t\)\( (*\ \[CapitalADoubleDot]quivalent\
\ zu\ MatrixForm[%]\ *) \)\)\)], "Input",
  CellLabel->"In[102]:="],

Cell[BoxData[
    \(\(\(m . a\)\(\t\t\t\)\( (*\ 
      Lineare\ Abbildung\ eines\ Vektors\ *) \)\)\)], "Input",
  CellLabel->"In[103]:="],

Cell[TextData[{
  "Um Matrizen zu erstellen gibt es nat\[UDoubleDot]rlich ",
  Cell[BoxData[
      \(Table\)]],
  ", aber zus\[ADoubleDot]tzlich noch einige spezielle Funktionen. Eine 3\
\[Times]3 Matrix erstellen..."
}], "Text"],

Cell[BoxData[
    \(b = Table[i*j, {i, 3}, {j, 3}]\)], "Input",
  CellLabel->"In[104]:="],

Cell["... oder eine 3\[Times]3 Diagonalmatrix...", "Text"],

Cell[BoxData[
    \(d = DiagonalMatrix[{x1, x2, x3}]\)], "Input",
  CellLabel->"In[105]:="],

Cell["... oder eine 3\[Times]3 Einheitsmatrix", "Text"],

Cell[BoxData[
    \(IdentityMatrix[3]\)], "Input",
  CellLabel->"In[106]:="],

Cell[TextData[{
  Cell[BoxData[
      \(b\)]],
  " ist symmetrisch, stimmt als mit der eigenen Transponierten \
\[UDoubleDot]berein"
}], "Text"],

Cell[BoxData[
    \(Transpose[b] \[Equal] b\)], "Input",
  CellLabel->"In[107]:="],

Cell["\<\
Andere klassische Funktionen einer Matrix sind die Determinante und \
die Inverse\
\>", "Text"],

Cell[BoxData[
    \(Det[d]\)], "Input",
  CellLabel->"In[108]:="],

Cell[BoxData[
    \(Inverse[d]\)], "Input",
  CellLabel->"In[109]:="],

Cell[BoxData[
    \(Clear[a, b, d, m]\)], "Input",
  CellLabel->"In[110]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Lineare Gleichungssysteme"
}], "Subsection"],

Cell[TextData[{
  "Lineare Gleichungssysteme k\[ODoubleDot]nnen nat\[UDoubleDot]rlich mit den \
allgemeinen L\[ODoubleDot]sungsverfahren von ",
  Cell[BoxData[
      \(Solve\)]],
  " aufgel\[ODoubleDot]st werden. Das ist der praktischere Weg, wenn bereits \
die Gleichungen selbst vorliegen. Wenn dagegen nur die Matrix ",
  Cell[BoxData[
      \(m\)]],
  " und der Vektor ",
  Cell[BoxData[
      \(b\)]],
  " in der Gleichung ",
  Cell[BoxData[
      \(m . x \[Equal] b\)]],
  " bekannt sind, ist es einfacher, die Funktion ",
  Cell[BoxData[
      \(LinearSolve\)]],
  " zu verwenden:"
}], "Text"],

Cell[BoxData[
    \(LinearSolve[{{1, 2}, {3, 4}}, {a, b}]\)], "Input",
  CellLabel->"In[111]:="],

Cell[TextData[{
  "F\[UDoubleDot]r homogene LGSe kann man auch zu ",
  Cell[BoxData[
      \(NullSpace\)]],
  " greifen, um eine Basis des Kerns der Matrix zu ermitteln"
}], "Text"],

Cell[BoxData[
    \(NullSpace[{{2, \(-2\)}, {4, \(-4\)}}]\)], "Input",
  CellLabel->"In[112]:="],

Cell[TextData[{
  "Muss ein lineares Gleichungssystem mehrfach f\[UDoubleDot]r verschiedene \
",
  Cell[BoxData[
      \(b\)]],
  " aber konstantes ",
  Cell[BoxData[
      \(m\)]],
  " gel\[ODoubleDot]st werden, kann es effizienter sein, zun\[ADoubleDot]chst \
eine LU-Dekomposition der Matrix durchzuf\[UDoubleDot]hren:"
}], "Text"],

Cell[BoxData[
    \(lu = LUDecomposition[{{1, 2}, {3, 4}}]\)], "Input",
  CellLabel->"In[113]:="],

Cell[BoxData[
    \(LUBackSubstitution[lu, {a, b}]\)], "Input",
  CellLabel->"In[114]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Eigenwerte und Eigenvektoren"
}], "Subsection"],

Cell[TextData[{
  "Zur Berechnung der Eigenwerte und -vektoren einer Matrix stellt \
Mathematica die Funktionen ",
  Cell[BoxData[
      \(Eigenvalues\)]],
  " und ",
  Cell[BoxData[
      \(Eigenvectors\)]],
  " zur Verf\[UDoubleDot]gung. F\[UDoubleDot]r eine ",
  Cell[BoxData[
      \(n\[Times]n\)]],
  " Matrix liefert ",
  Cell[BoxData[
      \(Eigenvalues\)]],
  " immer ",
  Cell[BoxData[
      \(n\)]],
  " (nicht unbedingt verschiedene) Zahlen, w\[ADoubleDot]hrend ",
  Cell[BoxData[
      \(Eigenvectors\)]],
  " bei weniger als ",
  Cell[BoxData[
      \(n\)]],
  " ",
  StyleBox["unabh\[ADoubleDot]ngigen",
    FontSlant->"Italic"],
  " Eigenvektoren (wie im folgenden Beispiel) Nullvektoren \
anh\[ADoubleDot]ngt."
}], "Text"],

Cell[BoxData[
    \(\(m = {{2, 1, 0}, {0, 3, \(-1\)}, {0, 0, 3}};\)\)], "Input",
  CellLabel->"In[115]:="],

Cell[BoxData[
    \(Eigenvalues[m]\)], "Input",
  CellLabel->"In[116]:="],

Cell[BoxData[
    \(Eigenvectors[m]\)], "Input",
  CellLabel->"In[117]:="],

Cell[TextData[{
  Cell[BoxData[
      \(Eigensystem\)]],
  " berechnet gleichzeitig die Eigenwerte und -vektoren einer Matrix und gibt \
sie als Liste zur\[UDoubleDot]ck."
}], "Text"],

Cell[BoxData[
    \({ew, ev} = Eigensystem[m]\)], "Input",
  CellLabel->"In[118]:="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["5. Grafik", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Zweidimensional"
}], "Subsection"],

Cell[TextData[{
  Cell[BoxData[
      \(Plot\)]],
  " stellt eine Funktion auf einem reellen Intervall dar\[Ellipsis]"
}], "Text"],

Cell[BoxData[
    \(\(Plot[x\^2/Log[x], {x, 0, 10}];\)\)], "Input",
  CellLabel->"In[119]:="],

Cell[TextData[{
  "\[Ellipsis] oder auch mehrere Funktionen. Das folgende Beispiel zeigt \
auch, wie man eine ",
  StyleBox["Option",
    FontSlant->"Italic"],
  " an ",
  Cell[BoxData[
      \(Plot\)]],
  " \[UDoubleDot]bergibt (hier ",
  Cell[BoxData[
      \(Frame \[Rule] True\)]],
  ")."
}], "Text"],

Cell[BoxData[
    \(\(Plot[{BesselJ[0, x], BesselJ[1, x]}, {x, 0, 10}, 
        Frame \[Rule] True];\)\)], "Input",
  CellLabel->"In[120]:="],

Cell[TextData[{
  "Parametrische Kurven in der Ebene werden von ",
  Cell[BoxData[
      \(ParametricPlot\)]],
  " dargestellt"
}], "Text"],

Cell[BoxData[
    \(\(ParametricPlot[{Cos[t]/t, Sin[t]/t}, {t, 0, 3  Pi}, 
        AspectRatio \[Rule] Automatic, ImageSize \[Rule] 200];\)\)], "Input",
  CellLabel->"In[121]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Dreidimensional"
}], "Subsection"],

Cell[TextData[{
  "Eine Funktion ",
  Cell[BoxData[
      \(\[DoubleStruckCapitalR]\^2 \[Rule] \[DoubleStruckCapitalR]\)]],
  " kann mit ",
  Cell[BoxData[
      \(Plot3D\)]],
  " veranschaulicht werden"
}], "Text"],

Cell[BoxData[
    \(\(Plot3D[\((x\^2 + 2  y\^2)\) E\^\(1 - x^2 - y^2\), {x, \(-2\), 
          2}, {y, \(-2\), 2}, ImageSize \[Rule] 200];\)\)], "Input",
  CellLabel->"In[122]:="],

Cell[TextData[{
  "Parametrische Plots im Raum sind entweder Kurven, wenn nur ein Parameter \
verwendet wird, oder Oberfl\[ADoubleDot]chen f\[UDoubleDot]r zwei Parameter. \
Das zweite Beispiel zeigt den ",
  "Betrag der Kugelfl\[ADoubleDot]chenfunktion ",
  Cell[BoxData[
      \(Y\_2\^\(-1\)\)]]
}], "Text"],

Cell[BoxData[
    \(\(ParametricPlot3D[{Cos[t], Sin[t], t/4}, {t, 0, 2  Pi}, 
        ImageSize \[Rule] 200];\)\)], "Input",
  CellLabel->"In[123]:="],

Cell[BoxData[
    \(\(r[t_, p_] := Abs[SphericalHarmonicY[2, \(-1\), t, p]];\)\)], "Input",
  CellLabel->"In[124]:="],

Cell[BoxData[
    \(\(ParametricPlot3D[
        r[\[Theta], \[Phi]]\ {Sin[\[Theta]] Cos[\[Phi]], 
            Sin[\[Theta]] Sin[\[Phi]], 
            Cos[\[Theta]]}, \[IndentingNewLine]{\[Theta], 0, Pi}, {\[Phi], 0, 
          2  Pi}, AspectRatio \[Rule] 1, ImageSize \[Rule] 200];\)\)], "Input",\

  CellLabel->"In[125]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Andere Visualisierungsm\[ODoubleDot]glichkeiten"
}], "Subsection"],

Cell[TextData[{
  "Diskrete Punktmengen werden mit ",
  Cell[BoxData[
      \(ListPlot\)]],
  " gezeichnet. Die Option ",
  Cell[BoxData[
      \(PlotJoined \[Rule] True\)]],
  " verbindet die einzelnen Punkte."
}], "Text"],

Cell[BoxData[
    \(\(ListPlot[Table[{i, Log[Fibonacci[i]]}, {i, 50, 100}]];\)\)], "Input",
  CellLabel->"In[126]:="],

Cell[TextData[{
  Cell[BoxData[
      \(ContourPlot\)]],
  " bietet eine weitere M\[ODoubleDot]glichkeit, eine Funktion ",
  Cell[BoxData[
      \(\[DoubleStruckCapitalR]\^2 \[Rule] \[DoubleStruckCapitalR]\)]],
  " darzustellen:"
}], "Text"],

Cell[BoxData[
    \(\(ContourPlot[Sin[x\ y], {x, \(-1\), 1}, {y, \(-1\), 1}, 
        ImageSize \[Rule] 150];\)\)], "Input",
  CellLabel->"In[127]:="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["7. Einige vollst\[ADoubleDot]ndige Beispiele", "Section"],

Cell[TextData[{
  "Die folgenden Abschnitte demonstrieren, wie man ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " in einigen einfachen konkreten Problemen einsetzt. F\[UDoubleDot]r die \
Grafiken verwenden wir dabei durchgehend folgende Optionen:"
}], "Text"],

Cell[BoxData[
    \(\(SetOptions[{ListPlot, \ ParametricPlot, ContourPlot}, 
        ImageSize \[Rule] 150, \ AspectRatio \[Rule] Automatic];\)\)], "Input",\

  CellLabel->"In[128]:="],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Darstellung der n-ten Einheitswurzeln"
}], "Subsection"],

Cell[TextData[{
  "Die L\[ODoubleDot]sungen von Gleichungen liefert ",
  StyleBox["Solve",
    FontFamily->"Courier"],
  " in Form einer Liste von Ersetzungsregeln zur\[UDoubleDot]ck:"
}], "Text"],

Cell[BoxData[
    \(Solve[z^4 \[Equal] 1, z]\)], "Input",
  CellLabel->"In[129]:="],

Cell[TextData[{
  "Um eine Liste aller n-ten Einheitswurzeln (aller L\[ODoubleDot]sungen der \
Gleichung ",
  Cell[BoxData[
      \(z\^n \[Equal] 1\)]],
  ") zu bestimmen, kann man also schreiben"
}], "Text"],

Cell[BoxData[
    \(einheitsw[n_] := Module[{z}, z /. Solve[z^n \[Equal] 1, z]]\)], "Input",\

  CellLabel->"In[130]:="],

Cell[TextData[{
  "Separiert man Real- und Imagin\[ADoubleDot]rteil erh\[ADoubleDot]lt man \
hieraus die entsprechende Punktmenge in der Gau\[SZ]schen Zahlenebene, die \
man schlie\[SZ]lich mit ",
  StyleBox["ListPlot",
    FontFamily->"Courier"],
  " darstellen kann"
}], "Text"],

Cell[BoxData[
    \(ewxy = Map[{Re[#], Im[#]} &, einheitsw[8]]\)], "Input",
  CellLabel->"In[131]:="],

Cell[BoxData[
    \(\(ListPlot[ewxy, 
        PlotStyle \[Rule] {AbsolutePointSize[4]}];\)\)], "Input",
  CellLabel->"In[132]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Koch-Kurve (zum Knobeln)"
}], "Subsection"],

Cell["\<\
Dieses Beispiel demonstriert die rekursive Konstruktion einer \
Koch-Kurve. Ausgangsfigur ist dabei das folgende gleichseitige Dreieck.\
\>", \
"Text"],

Cell[BoxData[
    \(\(dreieck = 
        List[{{\(-1\), 0}, {1, 0}}, {{1, 0}, {0, \@3}}, {{0, \@3}, {\(-1\), 
              0}}];\)\)], "Input",
  CellLabel->"In[133]:="],

Cell[TextData[{
  "Diese Funktion erstellt eine graphische Darstellung einer Koch\[Dash]Kurve \
n\[Dash]ter Ordnung. Die Kurve selbst wird wird von ",
  StyleBox["verfeinerung ",
    FontFamily->"Courier"],
  "berechnet, die obiges Dreieck in n rekursiven Schritten \"verfeinert\" und \
eine Kantenliste zur\[UDoubleDot]ckliefert."
}], "Text"],

Cell[BoxData[
    \(kochkurve[n_Integer] := 
      Graphics[Map[Line, verfeinerung[dreieck, n]]]\)], "Input",
  CellLabel->"In[134]:="],

Cell["Diese Verfeinerung l\[ADoubleDot]uft wie folgt ab:", "Text"],

Cell[BoxData[{
    \(\(verfeinerung[kanten_, \ 0] := kanten;\)\), "\[IndentingNewLine]", 
    \(verfeinerung[kanten_List, \ n_Integer] := 
      verfeinerung[\[IndentingNewLine]Flatten[Map[auswoelben, kanten], 
          1], \[IndentingNewLine]n - 1]\)}], "Input",
  CellLabel->"In[135]:="],

Cell["\<\
Schlie\[SZ]lich das Ausw\[ODoubleDot]lben einer Kante: Das mittlere \
Drittel jeder Kante wird entfernt und in die L\[UDoubleDot]cke die Spitze \
eines kleineren gleichseitigen Dreiecks eingesetzt\
\>", "Text"],

Cell[BoxData[{
    \(<< Geometry`Rotations`\), "\[IndentingNewLine]", 
    \(rotmat = RotationMatrix2D[60  Degree]\)}], "Input",
  CellLabel->"In[137]:="],

Cell[BoxData[
    \(auswoelben[{a_, b_}] := 
      Module[{x, y, s}, \[IndentingNewLine]x = 
          a + \((b - a)\)/3; \[IndentingNewLine]y = 
          b - \((b - a)\)/3; \[IndentingNewLine]s = 
          x + rotmat . \((y - x)\); \[IndentingNewLine]List[{a, x}, {x, 
            s}, {s, y}, {y, b}]]\)], "Input",
  CellLabel->"In[139]:="],

Cell[BoxData[
    \(GraphicsArray[Table[kochkurve[i], {i, 0, 4}]] // Show\)], "Input",
  CellLabel->"In[140]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Subsection"],
  ". Symmetrische Matrizen"
}], "Subsection"],

Cell[TextData[{
  "Reelle symmetrische Matrizen treten in vielen Zusammenh\[ADoubleDot]ngen \
auf, etwa als Kovarianzmatrix ",
  Cell[BoxData[
      \(\[CapitalSigma]\)]],
  " in mehrdimensionalen Normalverteilungen"
}], "Text"],

Cell[BoxData[
    \(p \((x)\) = 
      A\ e\^\(\(\(-1\)\/2\) \(\((x - \[Mu])\)\^T\) \
\(\[CapitalSigma]\^\(-1\)\) \((x - \[Mu])\)\)\)], "DisplayFormula"],

Cell[TextData[{
  "oder als Tr\[ADoubleDot]gheitstensor ",
  Cell[BoxData[
      \(J\)]],
  " bei der Definition der Rotationsenergie in der klassischen Mechanik"
}], "Text"],

Cell[BoxData[
    \(\(E\_rot\) \((\[Omega])\) = \(1\/2\) \(\[Omega]\^T\) 
        J\ \[Omega]\)], "DisplayFormula"],

Cell[TextData[{
  "In beiden F\[ADoubleDot]llen sind die mithilfe der Matrizen gebildeten \
Ausdr\[UDoubleDot]cke positiv definite ",
  StyleBox["quadratische Formen.",
    FontSlant->"Italic"],
  " Diese haben die besondere Eigenschaft, dass Punkte mit konstantem \
Funktionswert auf Hyperellipsoiden im jeweiligen Vektorraum liegen (etwa der \
Tr\[ADoubleDot]gheitsellipsoid der \[Omega]-Vektoren mit konstanter \
Rotationsenergie). Das kann man im Zweidimensionalen folgenderma\[SZ]en \
veranschaulichen."
}], "Text"],

Cell[BoxData[
    \(quadform[mtx_, vec_] := vec . mtx . vec\)], "Input",
  CellLabel->"In[141]:="],

Cell[BoxData[
    \(\(\(m = {{2, 3}, {3, 6}};\)\(\t\)\( (*\ 
      symmetrisch\ und\ pos . \ def . \ *) \)\)\)], "Input",
  CellLabel->"In[142]:="],

Cell[TextData[{
  "Einen \[CapitalUDoubleDot]berblick \[UDoubleDot]ber das Aussehen der zugeh\
\[ODoubleDot]rigen quadratischen Form kann man sich mit ",
  StyleBox["ContourPlot",
    FontFamily->"Courier"],
  " verschaffen"
}], "Text"],

Cell[BoxData[
    \(\(ContourPlot[
        quadform[m, {x, y}], {x, \(-10\), 10}, {y, \(-10\), 
          10}, \[IndentingNewLine]PlotPoints \[Rule] 40, 
        ColorFunction \[Rule] Hue, Contours \[Rule] 30];\)\)], "Input",
  CellLabel->"In[143]:="],

Cell["\<\
Wir dr\[UDoubleDot]cken die quadratische Form gleich in \
Polarkoordinaten aus, um eine der Ellipsen einfach parametrisieren zu k\
\[ODoubleDot]nnen \
\>", "Text"],

Cell[BoxData[
    \(qform[r_, t_] = 
      Simplify[quadform[m, {r\ Cos[t], r\ Sin[t]}]]\)], "Input",
  CellLabel->"In[144]:="],

Cell["\<\
Die Menge der Vektoren in der Ebene f\[UDoubleDot]r die die \
Funktion konstant Eins ist, l\[ADoubleDot]sst sich dann wie folgt \
darstellen\
\>", "Text"],

Cell[BoxData[
    \(r[t_] = Part[r /. Solve[qform[r, t] \[Equal] 1, r], 2]\)], "Input",
  CellLabel->"In[145]:="],

Cell[BoxData[
    \(g = 
      ParametricPlot[{r[t]*Cos[t], r[t]*Sin[t]}, {t, 0, 2  Pi}]\)], "Input",
  CellLabel->"In[146]:="],

Cell["\<\
Schlie\[SZ]lich kann man noch die Eigenvektoren von m einblenden. \
Wie man sieht, sind die Eigenvektoren gerade die Hauptachsen der Ellipse, und \
die L\[ADoubleDot]nge der Hauptachsen ist jeweils der Kehrwert der Wurzel aus \
dem zugeh\[ODoubleDot]rigen Eigenwert (warum?).\
\>", "Text"],

Cell[BoxData[
    \({ew, ev} = Eigensystem[m]\)], "Input",
  CellLabel->"In[147]:="],

Cell[BoxData[
    \(\(\(ev2 = Map[#/Sqrt[# . #] &, ev];\)\( (*\ 
      Eigenv . \ normieren\ *) \)\)\)], "Input",
  CellLabel->"In[148]:="],

Cell[BoxData[
    \(\(li = 
        Line[{ev2[\([1]\)]\/\@ew[\([1]\)], {0, 0}, 
            ev2[\([2]\)]\/\@ew[\([2]\)]}];\)\)], "Input",
  CellLabel->"In[149]:="],

Cell[BoxData[
    \(Show[{g, Graphics[{AbsoluteThickness[3], li}]}]\)], "Input",
  CellLabel->"In[150]:="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.1 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
ScreenStyleEnvironment->"Working",
WindowToolbars->{"RulerBar", "EditBar"},
WindowSize->{798, 572},
WindowMargins->{{Automatic, 95}, {Automatic, 20}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{597.562, 842.375},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "kuehn", \
"cp_phys", "notebooks"}, "mma-handb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1},
DefaultNaturalLanguage->Language,
Magnification->1,
StyleDefinitions -> Notebook[{
  
  Cell[CellGroupData[{
  Cell["Style Definitions", "Subtitle"],
  
  Cell["\<\
Modify the definitions below to change the default appearance of \
all cells in a given style.  Make modifications to any definition using \
commands in the Format menu.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell["Style Environment Names", "Subsection"],
  
  Cell[StyleData[All, "Working"],
    CellBracketOptions->{"Color"->RGBColor[0, 0.2, 1]},
    ScriptMinSize->9],
  
  Cell[StyleData[All, "Printout"],
    PageWidth->PaperWidth,
    PrivateFontOptions->{"FontType"->"Outline"}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Notebook Options", "Subsection"],
  
  Cell["\<\
  The options defined for the style below will be used at the \
Notebook level.  \
\>", "Text"],
  
  Cell[StyleData["Notebook"],
    PageHeaders->{{Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
            TextData[ {
              OptionValueBox[ "FileName"]}], "Header"]}, {Cell[ 
            TextData[ {
              OptionValueBox[ "FileName"]}], "Header"], None, Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"]}},
    PrintingOptions->{"FirstPageHeader"->False,
    "FacingPages"->True},
    CellLabelAutoDelete->False,
    CellFrameLabelMargins->6,
    StyleMenuListing->None]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Headings", "Subsection"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Title"],
    CellMargins->{{12, Inherited}, {2, 20}},
    CellGroupingRules->{"TitleGrouping", 0},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                  BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
        "MMA"->"Mathematica"},
    CounterIncrements->"Title",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subtitle", 0}, {"Subsubtitle", 0}},
    FontFamily->"Helvetica",
    FontSize->30,
    FontWeight->"Bold"],
  
  Cell[StyleData["Title", "Printout"],
    CellMargins->{{2, 10}, {2, 30}},
    FontSize->24]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subtitle"],
    CellMargins->{{12, Inherited}, {2, 2}},
    CellGroupingRules->{"TitleGrouping", 10},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                  BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
        "MMA"->"Mathematica"},
    CounterIncrements->"Subtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subsubtitle", 0}},
    FontFamily->"Helvetica",
    FontSize->24,
    FontColor->RGBColor[0, 0, 0.6]],
  
  Cell[StyleData["Subtitle", "Printout"],
    CellMargins->{{2, 10}, {2, 4}},
    FontSize->18,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubtitle"],
    CellMargins->{{12, Inherited}, {2, 12}},
    CellGroupingRules->{"TitleGrouping", 20},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                  BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
        "MMA"->"Mathematica"},
    CounterIncrements->"Subsubtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontSlant->"Italic",
    FontColor->RGBColor[0, 0, 0.6]],
  
  Cell[StyleData["Subsubtitle", "Printout"],
    CellMargins->{{2, 10}, {2, 8}},
    FontSize->12,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Section"],
    CellFrame->{{0, 0}, {2, 0}},
    CellMargins->{{10, 4}, {2, 50}},
    CellElementSpacings->{"ClosedGroupTopMargin"->18},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                  BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
        "MMA"->"Mathematica"},
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->18,
    FontWeight->"Bold"],
  
  Cell[StyleData["Section", "Printout"],
    CellMargins->{{2, 4}, {2, 80}},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsection"],
    CellDingbat->"\[FilledSquare]",
    CellMargins->{{24, 4}, {2, 18}},
    CellElementSpacings->{"ClosedGroupTopMargin"->12},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CellFrameLabelMargins->6,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                  BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
        "MMA"->"Mathematica"},
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->15,
    FontWeight->"Bold",
    CounterBoxOptions->{CounterStyle->"Subsection",
    CounterFunction:>RomanNumeral}],
  
  Cell[StyleData["Subsection", "Printout"],
    CellMargins->{{2, 4}, {2, 18}},
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                  BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
        "MMA"->"Mathematica"},
    FontSize->12]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubsection"],
    CellMargins->{{10, 4}, {2, 18}},
    CellElementSpacings->{"ClosedGroupTopMargin"->12},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                  BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
        "MMA"->"Mathematica"},
    CounterIncrements->"Subsubsection",
    FontFamily->"Helvetica",
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsubsection", "Printout"],
    CellMargins->{{2, 4}, {2, 18}},
    FontSize->10]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Text", "Subsection"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Text"],
    CellMargins->{{10, 4}, {0, 8}},
    InputAutoReplacements->{"TeX"->StyleBox[ 
            RowBox[ {"T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "LaTeX"->StyleBox[ 
            RowBox[ {"L", 
              StyleBox[ 
                AdjustmentBox[ 
                "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                  BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
              AdjustmentBox[ 
              "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
                0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
        "MMA"->"Mathematica"},
    TextJustification->1,
    Hyphenation->True,
    ParagraphSpacing->{0, 8},
    DefaultNaturalLanguage->"German",
    CounterIncrements->"Text"],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MathCaption"],
    CellFrame->{{4, 0}, {0, 0}},
    CellMargins->{{47, 62}, {0, 14}},
    CellFrameMargins->5,
    CellFrameColor->RGBColor[0, 0.2, 1],
    Hyphenation->True,
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 8},
    FontColor->RGBColor[0, 0, 0.6]],
  
  Cell[StyleData["MathCaption", "Printout"],
    CellMargins->{{34, 62}, {0, 14}},
    CellFrameColor->GrayLevel[0.700008],
    FontSize->10,
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Input/Output", "Subsection"],
  
  Cell["\<\
The cells in this section define styles used for input and output \
to the kernel.  Be careful when modifying, renaming, or removing these \
styles, because the front end associates special meanings with these style \
names.    \
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Input"],
    CellMargins->{{56, 4}, {3, 9}},
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    ShowCellLabel->True,
    CellLabelMargins->{{21, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultInputFormatType,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    LineSpacing->{1, 0},
    AutoItalicWords->{},
    LanguageCategory->"Formula",
    FormatType->StandardForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    LinebreakAdjustments->{0.85, 2, 10, 0, 1},
    CounterIncrements->"Input",
    FontWeight->"Bold"],
  
  Cell[StyleData["Input", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{42, 4}, {3, 8}},
    CellLabelMargins->{{21, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0},
    LinebreakAdjustments->{0.85, 2, 10, 1, 1},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Output"],
    CellMargins->{{57, 4}, {5, 2}},
    CellEditDuplicate->True,
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->True,
    CellLabelMargins->{{21, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    LineSpacing->{1, 0},
    AutoItalicWords->{},
    LanguageCategory->"Formula",
    FormatType->StandardForm,
    CounterIncrements->"Output"],
  
  Cell[StyleData["Output", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{42, 4}, {4, 2}},
    LineSpacing->{1, 0},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Message"],
    CellMargins->{{56, 4}, {3, 8}},
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    CellLabelMargins->{{21, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    AutoItalicWords->{},
    FormatType->InputForm,
    CounterIncrements->"Message",
    StyleMenuListing->None,
    FontColor->RGBColor[0, 0.2, 1]],
  
  Cell[StyleData["Message", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{42, 4}, {4, 2}},
    FontSize->10,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Print"],
    CellMargins->{{56, 4}, {3, 8}},
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    CellLabelMargins->{{21, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    AutoItalicWords->{},
    FormatType->InputForm,
    CounterIncrements->"Print",
    StyleMenuListing->None],
  
  Cell[StyleData["Print", "Printout"],
    ShowCellBracket->False,
    CellMargins->{{42, 4}, {4, 2}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Graphics"],
    CellMargins->{{56, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"GraphicsGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Graphics",
    StyleMenuListing->None],
  
  Cell[StyleData["Graphics", "Printout"],
    CellMargins->{{40, 4}, {4, 2}},
    ImageSize->{250, 250},
    FontSize->9]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["CellLabel"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontColor->RGBColor[0, 0.2, 1]],
  
  Cell[StyleData["CellLabel", "Printout"],
    FontSize->7,
    FontSlant->"Oblique",
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Unique Styles", "Subsection"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["TextTop"],
    CellFrame->{{0, 0}, {0, 2}},
    CellMargins->{{10, 4}, {2, 80}},
    CellHorizontalScrolling->True,
    CellFrameMargins->4,
    ShowSpecialCharacters->Automatic,
    ParagraphSpacing->{0, 8},
    CounterIncrements->"Text"],
  
  Cell[StyleData["TextTop", "Printout"],
    CellMargins->{{2, 4}, {2, 80}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ItemizedText"],
    CellMargins->{{20, 4}, {0, 8}},
    ShowSpecialCharacters->Automatic,
    Hyphenation->True,
    ParagraphSpacing->{0, 8},
    ParagraphIndent->-15,
    CounterIncrements->"Text"],
  
  Cell[StyleData["ItemizedText", "Printout"],
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ItemizedTextNote"],
    CellMargins->{{35, 4}, {0, 4}},
    ShowSpecialCharacters->Automatic,
    Hyphenation->True,
    ParagraphSpacing->{0, 4},
    CounterIncrements->"Text"],
  
  Cell[StyleData["ItemizedTextNote", "Printout"],
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["IndentedText"],
    CellMargins->{{20, 4}, {0, 6}},
    ShowSpecialCharacters->Automatic,
    Hyphenation->True,
    ParagraphSpacing->{0, 8},
    CounterIncrements->"Text"],
  
  Cell[StyleData["IndentedText", "Printout"],
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Note"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    ShowSpecialCharacters->Automatic,
    ParagraphSpacing->{0, 8},
    CounterIncrements->"Text",
    FontFamily->"Helvetica",
    FontSize->10],
  
  Cell[StyleData["Note", "Printout"],
    CellMargins->{{2, 4}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["1ColumnBox"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    LineIndent->0,
    Background->GrayLevel[0.8],
    FrameBoxOptions->{BoxMargins->{{1, 1}, {1.5, 1.5}}},
    GridBoxOptions->{ColumnSpacings->1}],
  
  Cell[StyleData["1ColumnBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["2ColumnBox"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    LineIndent->0,
    Background->GrayLevel[0.8],
    FrameBoxOptions->{BoxMargins->{{1, 1}, {1.5, 1.5}}},
    GridBoxOptions->{ColumnWidths->{0.39, 0.59}}],
  
  Cell[StyleData["2ColumnBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["2ColumnSmallBox"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    LineIndent->0,
    Background->GrayLevel[0.8],
    FrameBoxOptions->{BoxMargins->{{1, 1}, {1.5, 1.5}}},
    GridBoxOptions->{ColumnSpacings->1.5,
    ColumnWidths->0.35,
    ColumnAlignments->{Right, Left}}],
  
  Cell[StyleData["2ColumnSmallBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["3ColumnBox"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    LineIndent->0,
    Background->GrayLevel[0.8],
    FrameBoxOptions->{BoxMargins->{{1, 1}, {1.5, 1.5}}},
    GridBoxOptions->{ColumnWidths->0.325}],
  
  Cell[StyleData["3ColumnBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["3ColumnSmallBox"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    LineIndent->0,
    Background->GrayLevel[0.8],
    FrameBoxOptions->{BoxMargins->{{1, 1}, {1.5, 1.5}}},
    GridBoxOptions->{ColumnSpacings->1.5,
    ColumnWidths->0.23,
    ColumnAlignments->{Right, Center, Left}}],
  
  Cell[StyleData["3ColumnSmallBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["4ColumnBox"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    LineIndent->0,
    Background->GrayLevel[0.8],
    FrameBoxOptions->{BoxMargins->{{1, 1}, {1.5, 1.5}}},
    GridBoxOptions->{ColumnWidths->{0.145, 0.345, 0.145, 0.345}}],
  
  Cell[StyleData["4ColumnBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["5ColumnBox"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    LineIndent->0,
    Background->GrayLevel[0.8],
    FrameBoxOptions->{BoxMargins->{{1, 1}, {1.5, 1.5}}},
    GridBoxOptions->{ColumnWidths->0.195}],
  
  Cell[StyleData["5ColumnBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["6ColumnBox"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    LineIndent->0,
    Background->GrayLevel[0.8],
    FrameBoxOptions->{BoxMargins->{{1, 1}, {1.5, 1.5}}},
    GridBoxOptions->{ColumnWidths->{0.13, 0.23, 0.13, 0.13, 0.23, 0.13}}],
  
  Cell[StyleData["6ColumnBox", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10,
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Picture"],
    CellMargins->{{10, Inherited}, {0, 8}},
    CellHorizontalScrolling->True],
  
  Cell[StyleData["Picture", "Printout"],
    CellMargins->{{2, Inherited}, {0, 8}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Caption"],
    CellMargins->{{10, 50}, {0, 3}},
    PageBreakAbove->False,
    Hyphenation->True,
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["Caption", "Printout"],
    CellMargins->{{2, 50}, {2, 4}},
    FontSize->7]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Tables", "Subsection"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["2ColumnTable"],
    CellFrame->True,
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->False,
    LineBreakWithin->False,
    FontSize->10,
    GridBoxOptions->{ColumnSpacings->1.5,
    ColumnWidths->{0.39, 0.59},
    ColumnAlignments->{Right, Left}}],
  
  Cell[StyleData["2ColumnTable", "Printout"],
    CellFrame->True,
    CellMargins->{{2, 4}, {0, 8}}]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["3ColumnTable"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    StyleMenuListing->None,
    GridBoxOptions->{ColumnWidths->0.325,
    ColumnAlignments->{Left}}],
  
  Cell[StyleData["3ColumnTable", "Printout"],
    CellMargins->{{2, 4}, {0, 8}},
    FontSize->10]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Formulas and Programming", "Subsection"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ChemicalFormula"],
    CellMargins->{{42, Inherited}, {Inherited, Inherited}},
    CellHorizontalScrolling->True,
    DefaultFormatType->DefaultInputFormatType,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    LanguageCategory->"Formula",
    AutoSpacing->False,
    ScriptLevel->1,
    ScriptBaselineShifts->{0.6, Automatic},
    SingleLetterItalics->False,
    ZeroWidthTimes->True],
  
  Cell[StyleData["ChemicalFormula", "Printout"],
    CellMargins->{{34, Inherited}, {Inherited, Inherited}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DisplayFormula"],
    CellMargins->{{42, Inherited}, {Inherited, Inherited}},
    CellHorizontalScrolling->True,
    DefaultFormatType->DefaultInputFormatType,
    HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
    LanguageCategory->"Formula",
    ScriptLevel->0,
    SingleLetterItalics->True,
    SpanMaxSize->Infinity,
    UnderoverscriptBoxOptions->{LimitsPositioning->True},
    GridBoxOptions->{ColumnWidths->Automatic}],
  
  Cell[StyleData["DisplayFormula", "Printout"],
    CellMargins->{{34, Inherited}, {Inherited, Inherited}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Program"],
    CellMargins->{{10, 4}, {0, 8}},
    CellHorizontalScrolling->True,
    Hyphenation->False,
    LanguageCategory->"Formula",
    FontFamily->"Courier"],
  
  Cell[StyleData["Program", "Printout"],
    CellMargins->{{2, Inherited}, {Inherited, Inherited}},
    FontSize->9.5]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Hyperlink Styles", "Subsection"],
  
  Cell["\<\
The cells below define styles useful for making hypertext \
ButtonBoxes.  The \"Hyperlink\" style is for links within the same Notebook, \
or between Notebooks.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Hyperlink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonNote->ButtonData}],
  
  Cell[StyleData["Hyperlink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell["\<\
The following styles are for linking automatically to the on-line \
help system.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MainBookLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["MainBookLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["AddOnsLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["AddOnsLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["RefGuideLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "RefGuide", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["RefGuideLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["GettingStartedLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["GettingStartedLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["OtherInformationLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->RGBColor[0, 0, 1],
    FontVariations->{"Underline"->True},
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["OtherInformationLink", "Printout"],
    FontSize->10,
    FontColor->GrayLevel[0],
    FontVariations->{"Underline"->False}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Palette Styles", "Subsection"],
  
  Cell["\<\
The cells below define styles that define standard \
ButtonFunctions, for use in palette buttons.\
\>", "Text"],
  
  Cell[StyleData["Paste"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, After]}]&)}],
  
  Cell[StyleData["Evaluate"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, All], 
        SelectionEvaluate[ 
          FrontEnd`InputNotebook[ ], All]}]&)}],
  
  Cell[StyleData["EvaluateCell"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, All], 
        FrontEnd`SelectionMove[ 
          FrontEnd`InputNotebook[ ], All, Cell, 1], 
        FrontEnd`SelectionEvaluateCreateCell[ 
          FrontEnd`InputNotebook[ ], All]}]&)}],
  
  Cell[StyleData["CopyEvaluate"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`SelectionCreateCell[ 
          FrontEnd`InputNotebook[ ], All], 
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, All], 
        FrontEnd`SelectionEvaluate[ 
          FrontEnd`InputNotebook[ ], All]}]&)}],
  
  Cell[StyleData["CopyEvaluateCell"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`SelectionCreateCell[ 
          FrontEnd`InputNotebook[ ], All], 
        FrontEnd`NotebookApply[ 
          FrontEnd`InputNotebook[ ], #, All], 
        FrontEnd`SelectionEvaluateCreateCell[ 
          FrontEnd`InputNotebook[ ], All]}]&)}]
  }, Closed]]
  }, Open  ]]
  }]
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1727, 52, 97, 4, 61, "Title"],

Cell[CellGroupData[{
Cell[1849, 60, 47, 0, 85, "Section"],
Cell[1899, 62, 265, 4, 192, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2201, 71, 85, 4, 85, "Section"],

Cell[CellGroupData[{
Cell[2311, 79, 82, 4, 41, "Subsection"],
Cell[2396, 85, 702, 17, 72, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3135, 107, 109, 4, 41, "Subsection"],
Cell[3247, 113, 423, 11, 56, "Text"],
Cell[3673, 126, 62, 2, 27, "Input"],
Cell[3738, 130, 210, 6, 38, "Note"],
Cell[3951, 138, 258, 5, 40, "Text"],
Cell[4212, 145, 1009, 22, 89, "Text"],
Cell[5224, 169, 65, 2, 27, "Input"],
Cell[5292, 173, 185, 4, 40, "Text"],
Cell[5480, 179, 65, 2, 27, "Input"],
Cell[5548, 183, 68, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5653, 190, 148, 7, 41, "Subsection"],
Cell[5804, 199, 268, 7, 40, "Text"],
Cell[6075, 208, 92, 2, 27, "Input"],
Cell[6170, 212, 455, 11, 40, "Text"],
Cell[6628, 225, 73, 2, 27, "Input"],
Cell[6704, 229, 147, 3, 38, "Note"],
Cell[6854, 234, 623, 25, 56, "Text"],
Cell[7480, 261, 380, 10, 40, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7909, 277, 32, 0, 85, "Section"],

Cell[CellGroupData[{
Cell[7966, 281, 101, 4, 41, "Subsection"],
Cell[8070, 287, 803, 22, 72, "Text"],
Cell[8876, 311, 70, 2, 27, "Input"],
Cell[8949, 315, 61, 2, 27, "Input"],
Cell[9013, 319, 66, 2, 27, "Input"],
Cell[9082, 323, 66, 2, 27, "Input"],
Cell[9151, 327, 69, 2, 27, "Input"],
Cell[9223, 331, 565, 14, 56, "Text"],
Cell[9791, 347, 151, 4, 24, "Text"],
Cell[9945, 353, 201, 4, 59, "Input"],
Cell[10149, 359, 66, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10252, 366, 86, 4, 41, "Subsection"],
Cell[10341, 372, 596, 13, 80, "Text"],
Cell[10940, 387, 74, 2, 27, "Input"],
Cell[11017, 391, 80, 2, 27, "Input"],
Cell[11100, 395, 77, 2, 27, "Input"],
Cell[11180, 399, 70, 2, 27, "Input"],
Cell[11253, 403, 64, 2, 27, "Input"],
Cell[11320, 407, 74, 2, 27, "Input"],
Cell[11397, 411, 63, 2, 27, "Input"],
Cell[11463, 415, 101, 2, 27, "Input"],
Cell[11567, 419, 287, 10, 40, "Text"],
Cell[11857, 431, 78, 2, 27, "Input"],
Cell[11938, 435, 72, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12047, 442, 120, 5, 41, "Subsection"],
Cell[12170, 449, 76, 0, 24, "Text"],
Cell[12249, 451, 1728, 46, 105, "2ColumnTable"],
Cell[13980, 499, 68, 0, 24, "Text"],
Cell[14051, 501, 722, 18, 91, "2ColumnTable"],
Cell[14776, 521, 69, 2, 27, "Input"],
Cell[14848, 525, 67, 2, 27, "Input"],
Cell[14918, 529, 525, 16, 56, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[15492, 551, 50, 0, 85, "Section"],

Cell[CellGroupData[{
Cell[15567, 555, 77, 4, 41, "Subsection"],
Cell[15647, 561, 410, 10, 40, "Text"],
Cell[16060, 573, 74, 2, 27, "Input"],
Cell[16137, 577, 70, 2, 27, "Input"],
Cell[16210, 581, 63, 2, 27, "Input"],
Cell[16276, 585, 230, 6, 24, "Text"],
Cell[16509, 593, 76, 2, 27, "Input"],
Cell[16588, 597, 265, 9, 24, "Text"],
Cell[16856, 608, 98, 2, 27, "Input"],
Cell[16957, 612, 624, 18, 140, "Note"],
Cell[17584, 632, 552, 22, 40, "Text"],
Cell[18139, 656, 96, 2, 27, "Input"],
Cell[18238, 660, 267, 7, 40, "Text"],
Cell[18508, 669, 99, 2, 27, "Input"],
Cell[18610, 673, 191, 8, 24, "Text"],
Cell[18804, 683, 147, 3, 27, "Input"],
Cell[18954, 688, 176, 7, 24, "Text"],
Cell[19133, 697, 75, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19245, 704, 112, 4, 41, "Subsection"],
Cell[19360, 710, 259, 5, 40, "Text"],
Cell[19622, 717, 158, 3, 27, "Input"],
Cell[19783, 722, 64, 2, 27, "Input"],
Cell[19850, 726, 164, 5, 24, "Text"],
Cell[20017, 733, 108, 2, 27, "Input"],
Cell[20128, 737, 85, 2, 27, "Input"],
Cell[20216, 741, 397, 17, 40, "Text"],
Cell[20616, 760, 363, 12, 52, "Note"],
Cell[20982, 774, 109, 1, 24, "Text"],
Cell[21094, 777, 135, 3, 27, "Input"],
Cell[21232, 782, 143, 3, 27, "Input"],
Cell[21378, 787, 169, 4, 27, "Input"],
Cell[21550, 793, 464, 14, 40, "Text"],
Cell[22017, 809, 74, 2, 27, "Input"],
Cell[22094, 813, 81, 2, 27, "Input"],
Cell[22178, 817, 77, 0, 24, "Text"],
Cell[22258, 819, 90, 2, 27, "Input"],
Cell[22351, 823, 321, 15, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22709, 843, 130, 4, 41, "Subsection"],
Cell[22842, 849, 239, 6, 40, "Text"],
Cell[23084, 857, 84, 2, 27, "Input"],
Cell[23171, 861, 80, 2, 27, "Input"],
Cell[23254, 865, 206, 6, 24, "Text"],
Cell[23463, 873, 87, 2, 27, "Input"],
Cell[23553, 877, 95, 2, 27, "Input"],
Cell[23651, 881, 436, 13, 40, "Text"],
Cell[24090, 896, 77, 2, 27, "Input"],
Cell[24170, 900, 84, 2, 27, "Input"],
Cell[24257, 904, 108, 2, 27, "Input"],
Cell[24368, 908, 201, 8, 24, "Text"],
Cell[24572, 918, 111, 2, 27, "Input"],
Cell[24686, 922, 73, 2, 27, "Input"],
Cell[24762, 926, 416, 12, 40, "Text"],
Cell[25181, 940, 104, 2, 33, "Input"],
Cell[25288, 944, 226, 8, 24, "Text"],
Cell[25517, 954, 84, 2, 31, "Input"],
Cell[25604, 958, 97, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25738, 965, 99, 4, 41, "Subsection"],
Cell[25840, 971, 334, 10, 40, "Text"],
Cell[26177, 983, 91, 2, 27, "Input"],
Cell[26271, 987, 211, 5, 59, "Input"],
Cell[26485, 994, 174, 8, 24, "Text"],
Cell[26662, 1004, 356, 15, 41, "Text"],
Cell[27021, 1021, 90, 2, 27, "Input"],
Cell[27114, 1025, 87, 2, 27, "Input"],
Cell[27204, 1029, 73, 2, 27, "Input"],
Cell[27280, 1033, 75, 2, 27, "Input"],
Cell[27358, 1037, 77, 2, 27, "Input"],
Cell[27438, 1041, 155, 5, 24, "Text"],
Cell[27596, 1048, 94, 2, 27, "Input"],
Cell[27693, 1052, 184, 4, 59, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[27914, 1061, 90, 4, 41, "Subsection"],
Cell[28007, 1067, 1465, 34, 57, "Text",
  CounterIncrements->"Text"],
Cell[29475, 1103, 335, 8, 40, "Text"],
Cell[29813, 1113, 2178, 59, 167, "2ColumnTable"],
Cell[31994, 1174, 34, 0, 24, "Text"],
Cell[32031, 1176, 100, 2, 51, "Input"],
Cell[32134, 1180, 250, 5, 24, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[32433, 1191, 30, 0, 85, "Section"],

Cell[CellGroupData[{
Cell[32488, 1195, 102, 4, 41, "Subsection"],
Cell[32593, 1201, 95, 4, 24, "Text"],
Cell[32691, 1207, 73, 2, 27, "Input"],
Cell[32767, 1211, 88, 4, 24, "Text"],
Cell[32858, 1217, 77, 2, 27, "Input"],
Cell[32938, 1221, 110, 4, 24, "Text"],
Cell[33051, 1227, 76, 2, 27, "Input"],
Cell[33130, 1231, 40, 0, 24, "Text"],
Cell[33173, 1233, 86, 2, 44, "Input"],
Cell[33262, 1237, 202, 6, 24, "Text"],
Cell[33467, 1245, 92, 2, 27, "Input"],
Cell[33562, 1249, 67, 0, 24, "Text"],
Cell[33632, 1251, 93, 2, 27, "Input"],
Cell[33728, 1255, 123, 3, 24, "Text"],
Cell[33854, 1260, 120, 3, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34011, 1268, 90, 4, 41, "Subsection"],
Cell[34104, 1274, 294, 7, 40, "Text"],
Cell[34401, 1283, 83, 2, 27, "Input"],
Cell[34487, 1287, 87, 2, 27, "Input"],
Cell[34577, 1291, 123, 3, 24, "Text"],
Cell[34703, 1296, 64, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34804, 1303, 102, 4, 41, "Subsection"],
Cell[34909, 1309, 149, 3, 24, "Text"],
Cell[35061, 1314, 86, 2, 27, "Input"],
Cell[35150, 1318, 88, 2, 27, "Input"],
Cell[35241, 1322, 89, 2, 27, "Input"],
Cell[35333, 1326, 85, 2, 27, "Input"],
Cell[35421, 1330, 97, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[35567, 1338, 55, 0, 85, "Section"],

Cell[CellGroupData[{
Cell[35647, 1342, 106, 4, 41, "Subsection"],
Cell[35756, 1348, 222, 6, 24, "Text"],
Cell[35981, 1356, 95, 2, 27, "Input"],
Cell[36079, 1360, 304, 7, 40, "Text"],
Cell[36386, 1369, 104, 2, 27, "Input"],
Cell[36493, 1373, 357, 10, 40, "Text"],
Cell[36853, 1385, 93, 2, 27, "Input"],
Cell[36949, 1389, 160, 6, 24, "Text"],
Cell[37112, 1397, 81, 2, 27, "Input"],
Cell[37196, 1401, 62, 2, 27, "Input"],
Cell[37261, 1405, 143, 5, 24, "Text"],
Cell[37407, 1412, 88, 2, 27, "Input"],
Cell[37498, 1416, 155, 4, 24, "Text"],
Cell[37656, 1422, 143, 3, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37836, 1430, 94, 4, 41, "Subsection"],
Cell[37933, 1436, 191, 4, 24, "Text"],
Cell[38127, 1442, 107, 2, 27, "Input"],
Cell[38237, 1446, 142, 3, 27, "Input"],
Cell[38382, 1451, 386, 14, 40, "Text"],
Cell[38771, 1467, 190, 4, 27, "Input"],
Cell[38964, 1473, 381, 10, 40, "Text"],
Cell[39348, 1485, 145, 3, 27, "Input"],
Cell[39496, 1490, 142, 3, 24, "Text"],
Cell[39641, 1495, 86, 2, 27, "Input"],
Cell[39730, 1499, 197, 4, 40, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[39976, 1509, 37, 0, 85, "Section"],

Cell[CellGroupData[{
Cell[40038, 1513, 92, 4, 41, "Subsection"],
Cell[40133, 1519, 233, 4, 40, "Text"],
Cell[40369, 1525, 95, 2, 27, "Input"],
Cell[40467, 1529, 104, 2, 27, "Input"],
Cell[40574, 1533, 110, 2, 27, "Input"],
Cell[40687, 1537, 101, 2, 27, "Input"],
Cell[40791, 1541, 160, 3, 27, "Input"],
Cell[40954, 1546, 137, 3, 27, "Input"],
Cell[41094, 1551, 229, 6, 24, "Text"],
Cell[41326, 1559, 89, 2, 27, "Input"],
Cell[41418, 1563, 58, 0, 24, "Text"],
Cell[41479, 1565, 91, 2, 27, "Input"],
Cell[41573, 1569, 55, 0, 24, "Text"],
Cell[41631, 1571, 76, 2, 27, "Input"],
Cell[41710, 1575, 144, 5, 24, "Text"],
Cell[41857, 1582, 82, 2, 27, "Input"],
Cell[41942, 1586, 105, 3, 24, "Text"],
Cell[42050, 1591, 65, 2, 27, "Input"],
Cell[42118, 1595, 69, 2, 27, "Input"],
Cell[42190, 1599, 76, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42303, 1606, 96, 4, 41, "Subsection"],
Cell[42402, 1612, 600, 19, 56, "Text"],
Cell[43005, 1633, 96, 2, 27, "Input"],
Cell[43104, 1637, 181, 5, 24, "Text"],
Cell[43288, 1644, 96, 2, 27, "Input"],
Cell[43387, 1648, 334, 10, 40, "Text"],
Cell[43724, 1660, 97, 2, 27, "Input"],
Cell[43824, 1664, 89, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[43950, 1671, 99, 4, 41, "Subsection"],
Cell[44052, 1677, 739, 28, 56, "Text"],
Cell[44794, 1707, 106, 2, 27, "Input"],
Cell[44903, 1711, 73, 2, 27, "Input"],
Cell[44979, 1715, 74, 2, 27, "Input"],
Cell[45056, 1719, 183, 5, 24, "Text"],
Cell[45242, 1726, 84, 2, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[45375, 1734, 28, 0, 85, "Section"],

Cell[CellGroupData[{
Cell[45428, 1738, 86, 4, 41, "Subsection"],
Cell[45517, 1744, 130, 4, 24, "Text"],
Cell[45650, 1750, 93, 2, 31, "Input"],
Cell[45746, 1754, 304, 12, 24, "Text"],
Cell[46053, 1768, 141, 3, 27, "Input"],
Cell[46197, 1773, 139, 5, 24, "Text"],
Cell[46339, 1780, 178, 3, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46554, 1788, 86, 4, 41, "Subsection"],
Cell[46643, 1794, 215, 8, 24, "Text"],
Cell[46861, 1804, 179, 3, 31, "Input"],
Cell[47043, 1809, 308, 7, 40, "Text"],
Cell[47354, 1818, 150, 3, 27, "Input"],
Cell[47507, 1823, 117, 2, 27, "Input"],
Cell[47627, 1827, 324, 7, 43, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47988, 1839, 118, 4, 41, "Subsection"],
Cell[48109, 1845, 223, 8, 24, "Text"],
Cell[48335, 1855, 117, 2, 27, "Input"],
Cell[48455, 1859, 241, 7, 24, "Text"],
Cell[48699, 1868, 150, 3, 27, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[48898, 1877, 63, 0, 85, "Section"],
Cell[48964, 1879, 271, 6, 40, "Text"],
Cell[49238, 1887, 184, 4, 27, "Input"],

Cell[CellGroupData[{
Cell[49447, 1895, 108, 4, 41, "Subsection"],
Cell[49558, 1901, 196, 5, 24, "Text"],
Cell[49757, 1908, 83, 2, 27, "Input"],
Cell[49843, 1912, 208, 6, 24, "Text"],
Cell[50054, 1920, 120, 3, 27, "Input"],
Cell[50177, 1925, 280, 7, 40, "Text"],
Cell[50460, 1934, 101, 2, 27, "Input"],
Cell[50564, 1938, 129, 3, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50730, 1946, 95, 4, 41, "Subsection"],
Cell[50828, 1952, 161, 4, 24, "Text"],
Cell[50992, 1958, 170, 4, 33, "Input"],
Cell[51165, 1964, 343, 7, 40, "Text"],
Cell[51511, 1973, 135, 3, 27, "Input"],
Cell[51649, 1978, 66, 0, 24, "Text"],
Cell[51718, 1980, 290, 5, 75, "Input"],
Cell[52011, 1987, 220, 4, 40, "Text"],
Cell[52234, 1993, 154, 3, 43, "Input"],
Cell[52391, 1998, 343, 7, 91, "Input"],
Cell[52737, 2007, 112, 2, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52886, 2014, 92, 4, 41, "Subsection"],
Cell[52981, 2020, 228, 6, 24, "Text"],
Cell[53212, 2028, 153, 3, 31, "DisplayFormula"],
Cell[53368, 2033, 174, 5, 24, "Text"],
Cell[53545, 2040, 115, 2, 40, "DisplayFormula"],
Cell[53663, 2044, 520, 10, 56, "Text"],
Cell[54186, 2056, 98, 2, 27, "Input"],
Cell[54287, 2060, 147, 3, 27, "Input"],
Cell[54437, 2065, 236, 6, 24, "Text"],
Cell[54676, 2073, 251, 5, 43, "Input"],
Cell[54930, 2080, 173, 4, 24, "Text"],
Cell[55106, 2086, 127, 3, 27, "Input"],
Cell[55236, 2091, 164, 4, 24, "Text"],
Cell[55403, 2097, 113, 2, 27, "Input"],
Cell[55519, 2101, 127, 3, 27, "Input"],
Cell[55649, 2106, 299, 5, 40, "Text"],
Cell[55951, 2113, 84, 2, 27, "Input"],
Cell[56038, 2117, 139, 3, 27, "Input"],
Cell[56180, 2122, 163, 4, 50, "Input"],
Cell[56346, 2128, 106, 2, 27, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

